# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_policy import policy

from searchlight.common.policies import base

RESOURCE = 'resource:OS::%s'

rules = [
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Glance::Image',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Glance Image resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Glance::Metadef',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Glance Metadef resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Nova::Server',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Nova Server resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Nova::Hypervisor',
        check_str="rule:context_is_admin",
        description='Query with Nova Hypervisor resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Nova::ServerGroup',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Nova ServerGroup resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Nova::Flavor',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Nova Flavor resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Cinder::Volume',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Cinder Volume resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Cinder::Snapshot',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Cinder Snapshot resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Designate::Zone',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Designate Zone resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Designate::RecordSet',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Designate RecordSet resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Neutron::Net',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Neutron Net resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Neutron::Port',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Neutron Port resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Neutron::Subnet',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Neutron Subnet resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Neutron::Router',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Neutron Router resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Neutron::SecurityGroup',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Neutron SecurityGroup resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Ironic::Chassis',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Ironic Chassis resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Ironic::Node',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Ironic Node resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
    policy.DocumentedRuleDefault(
        name=RESOURCE % 'Ironic::Port',
        check_str=base.RULE_ADMIN_OR_OWNER,
        description='Query with Ironic Port resource.',
        operations=[
            {
                'path': '/v1/search',
                'method': 'POST'
            },
            {
                'path': '/v1/search',
                'method': 'GET'
            },
            {
                'path': '/v1/search/plugins',
                'method': 'GET'
            },
            {
                'path': '/v1/search/facets',
                'method': 'GET'
            }
        ]
    ),
]


def list_rules():
    return rules



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_concurrency import lockutils
from oslo_config import cfg

from searchlight.common import wsgi

CONF = cfg.CONF

_CACHED_THREAD_POOL = {}


def memoize(lock_name):
    def memoizer_wrapper(func):
        @lockutils.synchronized(lock_name)
        def memoizer(lock_name):
            if lock_name not in _CACHED_THREAD_POOL:
                _CACHED_THREAD_POOL[lock_name] = func()

            return _CACHED_THREAD_POOL[lock_name]

        return memoizer(lock_name)

    return memoizer_wrapper


def get_thread_pool(lock_name, size=1024):
    """Initializes eventlet thread pool.

    If thread pool is present in cache, then returns it from cache
    else create new pool, stores it in cache and return newly created
    pool.

    @param lock_name:  Name of the lock.
    @param size: Size of eventlet pool.

    @return: eventlet pool
    """
    @memoize(lock_name)
    def _get_thread_pool():
        return wsgi.get_asynchronous_eventlet_pool(size=size)

    return _get_thread_pool



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import mock

from oslo_utils import uuidutils

from searchlight.listener import NotificationEndpoint
from searchlight.pipeline import PipelineManager
from searchlight.tests import functional
from searchlight.tests.functional import test_api
from searchlight.tests.functional import test_listener
from searchlight.tests import utils

# These is in the load file
TENANT1 = "8eaac046b2c44ab99246cb0850c7f06d"
TENANT2 = "aaaaaabbbbbbccccc555552222255511"
TENANT3 = "75c31cdaa3604b76b7e279de50aec9f0"
TENANT4 = "1816a16093df465dbc609cf638422a05"

# Event tenant
EV_TENANT = "5fe7c4e4e492490393c674089a178e19"

NETID4 = "60e80dca-302d-4460-8984-9b85dc782bca"

SHARED_NET_ID = "deadbeef-4808-4c88-8c3b-deadbeefdead"


class TestNeutronPlugins(functional.FunctionalTest):
    def setUp(self):
        super(TestNeutronPlugins, self).setUp()
        self.networks_plugin = self.initialized_plugins['OS::Neutron::Net']
        self.subnets_plugin = self.initialized_plugins['OS::Neutron::Subnet']
        self.routers_plugin = self.initialized_plugins['OS::Neutron::Router']
        self.port_plugin = self.initialized_plugins['OS::Neutron::Port']
        self.fip_plugin = self.initialized_plugins['OS::Neutron::FloatingIP']
        self.secgroup_plugin = (
            self.initialized_plugins['OS::Neutron::SecurityGroup'])

        self.network_objects = self._load_fixture_data('load/networks.json')

        self.subnet_objects = self._load_fixture_data('load/subnets.json')
        self.subnet_objects = self.subnet_objects['subnets']

        self.port_objects = self._load_fixture_data('load/ports.json')
        self.port_objects = self.port_objects['ports']

        self.router_objects = self._load_fixture_data('load/routers.json')
        self.router_objects = self.router_objects['routers']

        self.fip_objects = self._load_fixture_data('load/floatingips.json')
        self.fip_objects = self.fip_objects['floatingips']

        self.secgroup_objects = self._load_fixture_data('load/security.json')
        self.secgroup_objects = self.secgroup_objects['securitygroups']

    def test_network_rbac_tenant(self):
        self._index(self.networks_plugin, self.network_objects)

        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      TENANT2)
        self.assertEqual(200, response.status_code)
        self.assertEqual(5, json_content['hits']['total'])

        hits = json_content['hits']['hits']
        expected_names = ['test-external-router', 'test-not-shared',
                          'test-shared', 'test1-no-shared-external',
                          'test1-shared-no-external']
        actual_names = [hit['_source']['name'] for hit in hits]

        self.assertEqual(set(expected_names), set(actual_names))

        response, json_content = self._search_request(
            {"query": {"match_all": {}}, "all_projects": True},
            TENANT2, role="admin")
        self.assertEqual(200, response.status_code)
        self.assertEqual(6, json_content['hits']['total'])

    def test_network_rbac_shared_external(self):
        """TENANT2 networks should be visible because they're marked
        shared or router:external
        """
        self._index(self.networks_plugin, self.network_objects)

        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      TENANT1)

        self.assertEqual(200, response.status_code)
        self.assertEqual(5, json_content['hits']['total'])

        hits = json_content['hits']['hits']
        expected_names = ['test', 'test-shared', 'test-external-router',
                          'test1-no-shared-external',
                          'test1-shared-no-external']
        actual_names = [hit['_source']['name'] for hit in hits]

        self.assertEqual(set(expected_names), set(actual_names))

    def test_subnet_rbac(self):
        """This test intentionally doesn't set a parent network so that it's
        just testing the tenant id check
        """
        self._index(self.subnets_plugin, self.subnet_objects)

        response, json_content = self._search_request(
            {"query": {"match_all": {}}, "type": "OS::Neutron::Subnet"},
            TENANT3)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        self.assertEqual(
            "doesnt-exist",
            json_content['hits']['hits'][0]['_source']['network_id'])

    def test_shared_network_subnet(self):
        self._index(self.networks_plugin, self.network_objects)
        self._index(self.subnets_plugin, self.subnet_objects)

        # There are two networks that belong to tenant2; shared-subnet and
        # not-shared-subnet. There's also a subnet called shared-not-external
        # that belongs to network id cf14a7a7-4808-4c88-8c3b-56fb48f2b1d1
        # and which is visible to TENANT2
        response, json_content = self._search_request(
            {"query": {"match_all": {}}, "type": "OS::Neutron::Subnet"},
            TENANT2)
        self.assertEqual(3, json_content['hits']['total'])
        self.assertEqual(
            set(['shared-subnet', 'not-shared-subnet', 'shared-not-external']),
            set([h['_source']['name'] for h in json_content['hits']['hits']]))

        # Tenant 3 can see the two shared network's subnets plus the one
        # that belongs to TENANT3
        response, json_content = self._search_request(
            {"query": {"match_all": {}}, "type": "OS::Neutron::Subnet"},
            TENANT3)
        self.assertEqual(3, json_content['hits']['total'])
        self.assertEqual(
            set(['shared-subnet', 'shared-not-external',
                 'ipv6-public-subnet']),
            set([h['_source']['name'] for h in json_content['hits']['hits']]))

    def test_subnet_rbac_admin_role_non_tenant(self):
        self._index(self.networks_plugin, self.network_objects)

        # Index all subnets
        self._index(self.subnets_plugin, self.subnet_objects)

        # There are now two subnets from two other tenant networks
        # which are either shared or external along with all subnets
        # from own tenant
        response, json_content = self._search_request(
            {"query": {"match_all": {}}, "type": "OS::Neutron::Subnet",
             "all_projects": False}, TENANT1, role="admin")
        self.assertEqual(4, json_content['hits']['total'])

    def test_port_rbac_admin_role_non_tenant(self):
        """Test that a user with admin role can access ports from
        all the tenants where a network is either shared or external.
        """
        self._index(self.networks_plugin, self.network_objects)

        self._index(self.port_plugin, self.port_objects)

        response, json_content = self._search_request(
            {"query": {"match_all": {}}, "type": "OS::Neutron::Port",
             "all_projects": False}, TENANT1, role="admin")
        self.assertEqual(200, response.status_code)
        self.assertEqual(2, json_content['hits']['total'])

    def test_router_rbac(self):
        self._index(self.routers_plugin,
                    self.router_objects)

        query = {"query": {"match_all": {}}, "type": "OS::Neutron::Router"}
        response, json_content = self._search_request(query,
                                                      TENANT3)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        self.assertEqual('0b143748-44d2-4545-9230-864c3abbc786',
                         json_content['hits']['hits'][0]['_source']['id'])

        response, json_content = self._search_request(query,
                                                      TENANT4)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        self.assertEqual('324d16fc-c381-4ea4-8f77-feda17cea1d7',
                         json_content['hits']['hits'][0]['_source']['id'])

    def test_router_fixed_ip(self):
        self._index(self.routers_plugin,
                    self.router_objects)

        query = {
            "query": {
                "nested": {
                    "path": "external_gateway_info",
                    "query": {
                        "term": {"external_gateway_info.enable_snat": True}
                    }
                }
            },
            "type": "OS::Neutron::Router"
        }
        response, json_content = self._search_request(query,
                                                      TENANT4)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])

        fixed_ip_path = "external_gateway_info.external_fixed_ips"
        ip_addr = "%s.ip_address" % fixed_ip_path
        net_id = "external_gateway_info.network_id"
        query = {
            "query": {
                "nested": {
                    "path": "external_gateway_info",
                    "query": {
                        "bool": {
                            "must": [
                                {"term": {net_id: "fake-router-network-id"}},
                                {"nested": {
                                    "path": fixed_ip_path,
                                    "query": {
                                        "term": {ip_addr: "2001:db8::1"}}
                                }}
                            ]
                        }
                    }
                }
            }
        }
        response, json_content = self._search_request(query,
                                                      TENANT4)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])

    def test_floatingip_rbac_admin(self):
        self._index(self.fip_plugin,
                    self.fip_objects)
        query = {"query": {"match_all": {}}, "type": "OS::Neutron::FloatingIP"}
        response, json_content = self._search_request(query,
                                                      TENANT3)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        self.assertEqual('f0ff4710-e6f6-42c3-b6af-1b15f91e84b0',
                         json_content['hits']['hits'][0]['_source']['id'])

        query['all_projects'] = True
        response, json_content = self._search_request(query, TENANT3,
                                                      role='admin')
        self.assertEqual(200, response.status_code)
        self.assertEqual(2, json_content['hits']['total'])

    def test_secgroup_rbac(self):
        self._index(self.secgroup_plugin,
                    self.secgroup_objects)
        query = {"query": {"match_all": {}},
                 "type": "OS::Neutron::SecurityGroup"}
        response, json_content = self._search_request(query, TENANT1)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        self.assertEqual('223c7074-e593-43f2-a0e6-f6d87ee8fb36',
                         json_content['hits']['hits'][0]['_source']['id'])

        response, json_content = self._search_request(query, TENANT3)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        self.assertEqual('57512cb7-aa80-9129-b8cc-ddc93612760a',
                         json_content['hits']['hits'][0]['_source']['id'])


class TestNeutronListeners(test_listener.TestSearchListenerBase):
    def __init__(self, *args, **kwargs):
        super(TestNeutronListeners, self).__init__(*args, **kwargs)
        self.network_events = self._load_fixture_data('events/networks.json')
        self.port_events = self._load_fixture_data('events/ports.json')
        self.subnet_events = self._load_fixture_data('events/subnets.json')
        self.router_events = self._load_fixture_data('events/routers.json')
        self.fip_events = self._load_fixture_data('events/floatingips.json')
        self.secgroup_events = self._load_fixture_data('events/security.json')

    def setUp(self):
        super(TestNeutronListeners, self).setUp()

        self.networks_plugin = self.initialized_plugins['OS::Neutron::Net']
        self.ports_plugin = self.initialized_plugins['OS::Neutron::Port']
        self.subnets_plugin = self.initialized_plugins['OS::Neutron::Subnet']
        self.routers_plugin = self.initialized_plugins['OS::Neutron::Router']
        self.fip_plugin = self.initialized_plugins['OS::Neutron::FloatingIP']
        self.secgroup_plugin = (
            self.initialized_plugins['OS::Neutron::SecurityGroup'])

        notification_plugins = {
            plugin.document_type: utils.StevedoreMock(plugin)
            for plugin in (self.networks_plugin, self.ports_plugin,
                           self.subnets_plugin, self.routers_plugin,
                           self.fip_plugin, self.secgroup_plugin)}
        self.notification_endpoint = NotificationEndpoint(
            notification_plugins,
            PipelineManager(notification_plugins)
        )

        self.listener_alias = self.networks_plugin.alias_name_listener

    def test_network_create_update_delete(self):
        '''Send network.create.end notification event to listener'''
        create_event = self.network_events['network.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)
        result = self._verify_event_processing(create_event, owner=EV_TENANT)
        verification_keys = ['id', 'status', 'port_security_enabled', 'name']
        self._verify_result(create_event, verification_keys, result,
                            inner_key='network')

        hit = result['hits']['hits'][0]['_source']
        self.assertEqual('2016-03-21T16:19:51', hit['updated_at'])

        update_event = self.network_events['network.update.end']
        self._send_event_to_listener(update_event, self.listener_alias)
        result = self._verify_event_processing(update_event, owner=EV_TENANT)
        verification_keys = ['id', 'status', 'port_security_enabled', 'name']
        self._verify_result(update_event, verification_keys, result,
                            inner_key='network')

        rbac_create_event = self.network_events['rbac_policy.create.end']
        self._send_event_to_listener(rbac_create_event, self.listener_alias)
        result = self._verify_event_processing(rbac_create_event,
                                               owner=EV_TENANT)
        cbody = result['hits']['hits'][0]['_source']
        # Verify internal states are not visible.
        self.assertNotIn('members', cbody)
        self.assertNotIn('rbac_policy', cbody)

        rbac_delete_event = self.network_events['rbac_policy.delete.end']
        self._send_event_to_listener(rbac_delete_event, self.listener_alias)
        result = self._verify_event_processing(rbac_delete_event,
                                               owner=EV_TENANT)
        dbody = result['hits']['hits'][0]['_source']
        # Verify internal states are not visible.
        self.assertNotIn('members', dbody)
        self.assertNotIn('rbac_policy', dbody)

        delete_event = self.network_events['network.delete.end']
        self._send_event_to_listener(delete_event, self.listener_alias)
        self._verify_event_processing(delete_event, count=0,
                                      owner=EV_TENANT)

    def test_port_create_event(self):
        create_event = self.port_events['port.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)
        result = self._verify_event_processing(create_event, owner=EV_TENANT)
        verification_keys = ['id', 'status', 'mac_address', 'status']
        self._verify_result(create_event, verification_keys, result,
                            inner_key='port')
        hit = result['hits']['hits'][0]['_source']
        self.assertEqual('2016-03-21T16:36:56', hit['updated_at'])

    def test_port_rename_event(self):
        update_event = self.port_events['port_rename']
        self._send_event_to_listener(update_event, self.listener_alias)
        result = self._verify_event_processing(update_event, owner=EV_TENANT)
        verification_keys = ['name']
        self._verify_result(update_event, verification_keys, result,
                            inner_key='port')

    def test_port_attach_detach_events(self):
        create_event = self.port_events['port.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)
        result = self._verify_event_processing(create_event, owner=EV_TENANT)
        verification_keys = ['device_owner', 'device_id']
        self._verify_result(create_event, verification_keys, result,
                            inner_key='port')

        attach_event = self.port_events['port_attach']
        self._send_event_to_listener(attach_event, self.listener_alias)
        result = self._verify_event_processing(attach_event, owner=EV_TENANT)
        verification_keys = ['device_owner', 'device_id']
        self._verify_result(attach_event, verification_keys, result,
                            inner_key='port')

        detach_event = self.port_events['port_detach']
        self._send_event_to_listener(detach_event, self.listener_alias)
        result = self._verify_event_processing(attach_event, owner=EV_TENANT)
        verification_keys = ['device_owner', 'device_id']
        self._verify_result(detach_event, verification_keys, result,
                            inner_key='port')

    def test_port_delete_event(self):
        delete_event = self.port_events['port.delete.end']
        self._send_event_to_listener(delete_event, self.listener_alias)
        self._verify_event_processing(None, count=0,
                                      owner=EV_TENANT)

    def test_subnet_create_update_delete(self):
        create_event = self.subnet_events['subnet.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)
        result = self._verify_event_processing(create_event, owner=EV_TENANT)
        verification_keys = ['network_id', 'name']
        self._verify_result(create_event, verification_keys, result,
                            inner_key='subnet')

        update_event = self.subnet_events['subnet.update.end']
        self._send_event_to_listener(update_event, self.listener_alias)
        result = self._verify_event_processing(update_event, owner=EV_TENANT)
        verification_keys = ['network_id', 'name']
        self._verify_result(update_event, verification_keys, result,
                            inner_key='subnet')

        delete_event = self.subnet_events['subnet.delete.end']
        self._send_event_to_listener(delete_event, self.listener_alias)
        self._verify_event_processing(delete_event, count=0,
                                      owner=EV_TENANT)

    def test_router_create_update_delete(self):
        create_event = self.router_events['router.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)
        result = self._verify_event_processing(create_event, owner=EV_TENANT)
        verification_keys = ['status', 'name', 'id']
        self._verify_result(create_event, verification_keys, result,
                            inner_key='router')

        update_event = self.router_events['router.update.end']
        self._send_event_to_listener(update_event, self.listener_alias)
        result = self._verify_event_processing(update_event, owner=EV_TENANT)
        verification_keys = ['status', 'name', 'id']
        self._verify_result(update_event, verification_keys, result,
                            inner_key='router')

        delete_event = self.router_events['router.delete.end']
        self._send_event_to_listener(delete_event, self.listener_alias)
        self._verify_event_processing(delete_event, count=0,
                                      owner=EV_TENANT)

    def test_router_interface_create_delete(self):
        """Check that port creation and deletion is registered on interface
        creation and deletion events
        """
        interface_port = {
            u'port': {
                u'admin_state_up': True,
                u'allowed_address_pairs': [],
                u'binding:vnic_type': u'normal',
                u'device_id': u'9262552b-6e46-41ee-9ede-393d5f65f325',
                u'device_owner': u'network:router_interface',
                u'dns_name': None,
                u'extra_dhcp_opts': [],
                u'fixed_ips': [{
                    u'ip_address': u'172.45.1.1',
                    u'subnet_id': u'4cd5c1d7-68ec-4e9a-bf16-9fd7571f8805'
                }],
                u'id': u'a5324522-47d3-4547-85df-a01ef6bde4b1',
                u'mac_address': u'fa:16:3e:5f:06:e3',
                u'name': u'',
                u'network_id': NETID4,
                u'port_security_enabled': False,
                u'security_groups': [],
                u'status': u'ACTIVE',
                u'tenant_id': EV_TENANT,
                u'created_at': '2016-03-17T18:54:23',
                u'updated_at': '2016-03-17T19:58:13'
            }
        }
        create_event = self.router_events['router.interface.create']
        with mock.patch('neutronclient.v2_0.client.Client.show_port',
                        return_value=interface_port):
            self._send_event_to_listener(create_event, self.listener_alias)

        query = {
            "type": "OS::Neutron::Port",
            "query": {"term": {"id": "a5324522-47d3-4547-85df-a01ef6bde4b1"}}
        }
        response, json_content = self._search_request(query,
                                                      EV_TENANT)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        hit = json_content['hits']['hits'][0]['_source']

        self.assertEqual(NETID4, hit['network_id'])
        self.assertEqual('network:router_interface', hit['device_owner'])

        delete_event = self.router_events['router.interface.delete']
        self._send_event_to_listener(delete_event, self.listener_alias)

        response, json_content = self._search_request(query,
                                                      EV_TENANT)
        self.assertEqual(0, json_content['hits']['total'])

    def test_ignore_neutron_dhcp_port(self):
        create_event = self.port_events['port.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)

        dhcp_event = copy.deepcopy(create_event)
        dhcp_event['payload']['port']['id'] = uuidutils.generate_uuid()
        dhcp_event['payload']['port']['device_owner'] = 'network:dhcp'
        self._send_event_to_listener(dhcp_event, self.listener_alias)

        query = {"type": "OS::Neutron::Port", "query": {"match_all": {}}}
        response, json_content = self._search_request(query,
                                                      EV_TENANT)
        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        self.assertEqual(create_event['payload']['port']['id'],
                         json_content['hits']['hits'][0]['_source']['id'])

    def test_floatingip_cud_events(self):
        create_event = self.fip_events['floatingip.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)
        result = self._verify_event_processing(create_event, owner=EV_TENANT)
        verification_keys = ['status', 'floating_network_id', 'id',
                             'fixed_ip_address', 'port_id']
        self._verify_result(create_event, verification_keys, result,
                            inner_key='floatingip')

        # Now associate it with a port
        associate_event = self.fip_events['floatingip.associate.end']
        self._send_event_to_listener(associate_event, self.listener_alias)
        result = self._verify_event_processing(associate_event,
                                               owner=EV_TENANT)
        verification_keys = ['status', 'floating_network_id', 'id',
                             'fixed_ip_address', 'port_id']
        self._verify_result(associate_event, verification_keys, result,
                            inner_key='floatingip')

        disassociate_event = self.fip_events['floatingip.disassociate.end']
        self._send_event_to_listener(disassociate_event, self.listener_alias)
        result = self._verify_event_processing(disassociate_event,
                                               owner=EV_TENANT)
        verification_keys = ['status', 'floating_network_id', 'id',
                             'fixed_ip_address', 'port_id']
        self._verify_result(disassociate_event, verification_keys, result,
                            inner_key='floatingip')

        delete_event = self.fip_events['floatingip.delete.end']
        self._send_event_to_listener(delete_event, self.listener_alias)
        self._verify_event_processing(delete_event, count=0,
                                      owner=EV_TENANT)

    def test_secgroup_create_delete(self):
        # Test #1: Create a security group.
        create_event = self.secgroup_events['security_group.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)

        query = {"query": {"match_all": {}},
                 "type": "OS::Neutron::SecurityGroup"}
        response, json_content = self._search_request(query, EV_TENANT)

        self.assertEqual(200, response.status_code)
        self.assertEqual(1, json_content['hits']['total'])
        self.assertEqual(create_event['payload']['security_group']['id'],
                         json_content['hits']['hits'][0]['_source']['id'])
        # Verify the security group has two security group rules.
        groups = (
            json_content['hits']['hits'][0]['_source']['security_group_rules'])
        self.assertEqual(2, len(groups))

        # Test #2: Create a new security group rule for security group.
        create_event = self.secgroup_events['security_group_rule.create.end']
        self._send_event_to_listener(create_event, self.listener_alias)

        query = {"query": {"match_all": {}},
                 "type": "OS::Neutron::SecurityGroup"}
        response, json_content = self._search_request(query, EV_TENANT)

        self.assertEqual(200, response.status_code)
        # Verify the new rule was added to the security group.
        groups = (
            json_content['hits']['hits'][0]['_source']['security_group_rules'])
        self.assertEqual(3, len(groups))

        # Test #3: Query security groups rules, using nested queries (Passing).
        rule_id = "c36445af-953d-47e4-85d2-2a6ac8fcba14"
        group_id = "223c7074-e593-43f2-a0e6-f6d87ee8fb36"
        query = {"query": {"nested": {
                 "path": "security_group_rules",
                 "query": {"term": {"security_group_rules.id": rule_id}}}}}
        response, json_content = self._search_request(query, EV_TENANT)
        self.assertEqual(200, response.status_code)

        group = json_content['hits']['hits']
        self.assertEqual(1, len(group))
        self.assertEqual(group_id, group[0]['_source']['id'])

        # Test #4: Query security groups rules, using nested queries (Failing).
        rule_id = "c36445af-953d-47e4-85d2-2a6ac8fcba15"  # Wrong last digit.
        query = {"query": {"nested": {
                 "path": "security_group_rules",
                 "query": {"term": {"security_group_rules.id": rule_id}}}}}
        response, json_content = self._search_request(query, EV_TENANT)
        self.assertEqual(200, response.status_code)

        group = json_content['hits']['hits']
        self.assertEqual(0, len(group))

        # Test #5: Delete the new security group rule from security group.
        create_event = self.secgroup_events['security_group_rule.delete.end']
        self._send_event_to_listener(create_event, self.listener_alias)

        query = {"query": {"match_all": {}},
                 "type": "OS::Neutron::SecurityGroup"}
        response, json_content = self._search_request(query, EV_TENANT)

        self.assertEqual(200, response.status_code)
        # Verify the new rule was deleted from the security group.
        groups = (
            json_content['hits']['hits'][0]['_source']['security_group_rules'])
        self.assertEqual(2, len(groups))

        # Test #6. Delete the security group.
        create_event = self.secgroup_events['security_group.delete.end']
        self._send_event_to_listener(create_event, self.listener_alias)

        query = {"query": {"match_all": {}},
                 "type": "OS::Neutron::SecurityGroup"}
        response, json_content = self._search_request(query, EV_TENANT)

        self.assertEqual(200, response.status_code)
        self.assertEqual(0, json_content['hits']['total'])



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils
import webob

from searchlight.api import versions
import searchlight.tests.utils as test_utils


class VersionsTest(test_utils.BaseTestCase):

    """Test the version information returned from the API service."""

    def test_get_version_list(self):
        req = webob.Request.blank('/', base_url='http://127.0.0.1:9393/')
        req.accept = 'application/json'
        self.config(bind_host='127.0.0.1', bind_port=9393, group='api')
        res = versions.Controller().index(req)
        self.assertEqual(300, res.status_int)
        self.assertEqual('application/json', res.content_type)
        results = jsonutils.loads(res.body)['versions']
        expected = [
            {
                'id': 'v1.0',
                'status': 'CURRENT',
                'links': [{'rel': 'self',
                           'href': 'http://127.0.0.1:9393/v1/'}],
            },
        ]
        self.assertEqual(expected, results)



# Copyright 2015 Intel Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from searchlight.api import policy
from searchlight.common import property_utils
from searchlight.common import resource_types
from searchlight.common import utils
from searchlight.elasticsearch.plugins import base
from searchlight.elasticsearch.plugins.glance \
    import images_notification_handler
from searchlight.elasticsearch.plugins.glance import serialize_glance_image


class ImageIndex(base.IndexBase):
    NotificationHandlerCls = images_notification_handler.ImageHandler

    def __init__(self, policy_enforcer=None):
        super(ImageIndex, self).__init__()
        self.policy = policy_enforcer or policy.Enforcer()
        if property_utils.is_property_protection_enabled():
            self.property_rules = property_utils.PropertyRules(self.policy)
        self._image_base_properties = [
            'checksum', 'created_at', 'container_format', 'disk_format', 'id',
            'min_disk', 'min_ram', 'name', 'size', 'virtual_size', 'status',
            'tags', 'updated_at', 'visibility', 'protected', 'owner',
            'members', 'project_id']

    @classmethod
    def get_document_type(cls):
        return resource_types.GLANCE_IMAGE

    def get_mapping(self):
        return {
            'dynamic': True,
            'properties': {
                'id': {'type': 'string', 'index': 'not_analyzed'},
                'kernel_id': {'type': 'string', 'index': 'not_analyzed'},
                'ramdisk_id': {'type': 'string', 'index': 'not_analyzed'},
                'name': {
                    'type': 'string',
                    'fields': {
                        'raw': {'type': 'string', 'index': 'not_analyzed'}
                    }
                },
                'description': {'type': 'string'},
                'tags': {'type': 'string'},
                'disk_format': {'type': 'string'},
                'container_format': {'type': 'string'},
                'size': {'type': 'long'},
                'virtual_size': {'type': 'long'},
                'status': {'type': 'string', 'index': 'not_analyzed'},
                'visibility': {'type': 'string', 'index': 'not_analyzed'},
                'checksum': {'type': 'string'},
                'min_disk': {'type': 'long'},
                'min_ram': {'type': 'long'},
                'owner': {'type': 'string', 'index': 'not_analyzed'},
                'project_id': {'type': 'string', 'index': 'not_analyzed'},
                'protected': {'type': 'boolean'},
                'members': {'type': 'string', 'index': 'not_analyzed'},
                'created_at': {'type': 'date'},
                'updated_at': {'type': 'date'},

                # For images based on snapshot
                'image_type': {'type': 'string', 'index': 'not_analyzed'},
                'base_image_ref': {'type': 'string', 'index': 'not_analyzed'},
                'image_state': {'type': 'string', 'index': 'not_analyzed'},
                'user_id': {'type': 'string', 'index': 'not_analyzed'},
                'instance_uuid': {'type': 'string', 'index': 'not_analyzed'}
            },
            "_meta": {
                "owner": {
                    "resource_type": resource_types.KEYSTONE_PROJECT
                },
                "project_id": {
                    "resource_type": resource_types.KEYSTONE_PROJECT
                },
                "kernel_id": {
                    "resource_type": resource_types.GLANCE_IMAGE
                },
                "ramdisk_id": {
                    "resource_type": resource_types.GLANCE_IMAGE
                },
                "base_image_ref": {
                    "resource_type": resource_types.GLANCE_IMAGE
                },
                "user_id": {
                    "resource_type": resource_types.KEYSTONE_USER
                },
                "instance_uuid": {
                    "resource_type": resource_types.NOVA_SERVER
                },
            }
        }

    @property
    def facets_with_options(self):
        return ('disk_format', 'container_format', 'tags', 'visibility',
                'protected', 'status', 'image_type', 'image_state')

    @property
    def facets_excluded(self):
        return {'owner': True, 'project_id': True}

    @property
    def resource_allowed_policy_target(self):
        return 'get_images'

    @property
    def service_type(self):
        return 'image'

    def _get_rbac_field_filters(self, request_context):
        return [
            {'term': {'owner': request_context.owner}},
            {'term': {'visibility': 'public'}},
            {'term': {'members': request_context.tenant}}
        ]

    def filter_result(self, hit, request_context):
        super(ImageIndex, self).filter_result(hit, request_context)
        highlight = hit.get('highlight')
        if highlight:
            utils.restore_dots_in_field_names(highlight)
        source = hit['_source']
        utils.restore_dots_in_field_names(source)
        if property_utils.is_property_protection_enabled():
            for key in list(source.keys()):
                if key not in self._image_base_properties:
                    if not self.property_rules.check_property_rules(
                            key, 'read', request_context):
                        del source[key]

        # Strip out the members list for non-admins or non-owners. The glance
        # api leaves just the current tenant in the list for non-admins
        if not (request_context.is_admin or
                source['owner'] == request_context.tenant):
            if source.get('members', []):
                # Remove any members that don't match the current tenant
                source['members'] = list(filter(
                    lambda m: m == request_context.tenant,
                    source['members']))

    def get_objects(self):
        from searchlight.elasticsearch.plugins import openstack_clients
        # Images include their properties and tags. Members are different
        return openstack_clients.get_glanceclient().images.list()

    def serialize(self, obj):
        return serialize_glance_image(obj)



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import random

from searchlight.common import utils
from searchlight.elasticsearch.plugins.neutron import\
    networks as networks_plugin
import searchlight.tests.unit.utils as unit_test_utils
import searchlight.tests.utils as test_utils


_now_str = utils.isotime(datetime.datetime.utcnow())
USER1 = u'27f4d76b-be62-4e4e-aa33bb11cc55'
ID1 = "813dd936-663e-4e5b-877c-986021b73e2c"
TENANT1 = "8eaac046b2c44ab99246cb0850c7f06d"


def _network_fixture(network_id, tenant_id, name, **kwargs):
    fixture = {
        "admin_state_up": True,
        "id": network_id,
        "mtu": 0,
        "name": name,
        "port_security_enabled": True,
        "provider:network_type": "vxlan",
        "provider:physical_network": None,
        "provider:segmentation_id": random.randint(1000, 1500),
        "router:external": False,
        "shared": False,
        "status": "ACTIVE",
        "subnets": [],
        "tenant_id": tenant_id,
        "updated_at": _now_str,
        "created_at": _now_str
    }
    fixture.update(kwargs)
    return fixture


class TestNetworkLoaderPlugin(test_utils.BaseTestCase):
    def setUp(self):
        super(TestNetworkLoaderPlugin, self).setUp()
        self.plugin = networks_plugin.NetworkIndex()
        self._create_fixtures()

    def _create_fixtures(self):
        self.network1 = _network_fixture(
            network_id=ID1, tenant_id=TENANT1, name="test-net-1")
        self.networks = [self.network1]

    def test_default_index_name(self):
        self.assertEqual('searchlight', self.plugin.resource_group_name)

    def test_document_type(self):
        self.assertEqual('OS::Neutron::Net',
                         self.plugin.get_document_type())

    def test_serialize(self):
        # Serialization doesn't do much right now; removes subnets
        serialized = self.plugin.serialize(self.network1)
        self.assertNotIn('subnets', serialized)
        self.assertEqual(_now_str, serialized['updated_at'])
        # project id should get copied from tenant_id
        self.assertEqual(TENANT1, serialized['project_id'])
        self.assertEqual([], serialized['members'])
        self.assertEqual([], serialized['rbac_policy'])

    def test_rbac_filter(self):
        fake_request = unit_test_utils.get_fake_request(
            USER1, TENANT1, '/v1/search', is_admin=False
        )
        rbac_terms = self.plugin._get_rbac_field_filters(fake_request.context)
        self.assertEqual(
            [
                {'term': {'tenant_id': TENANT1}},
                {'terms': {'members': [TENANT1, '*']}},
                {'term': {'router:external': True}},
                {'term': {'shared': True}}
            ],
            rbac_terms
        )

    def test_admin_only_fields(self):
        admin_only_fields = self.plugin.admin_only_fields
        self.assertEqual(['provider:*'], admin_only_fields)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from searchlight.common import resource_types
from searchlight.elasticsearch.plugins import base
from searchlight.elasticsearch.plugins.ironic import notification_handlers
from searchlight.elasticsearch.plugins.ironic import resources
from searchlight.elasticsearch.plugins.ironic import serialize_resource
from searchlight.elasticsearch.plugins import openstack_clients


class PortIndex(base.IndexBase):
    NotificationHandlerCls = notification_handlers.PortHandler

    @classmethod
    def parent_plugin_type(cls):
        return resource_types.IRONIC_NODE

    @classmethod
    def get_document_type(cls):
        return resource_types.IRONIC_PORT

    def get_mapping(self):
        return copy.deepcopy(resources.PORT_MAPPING)

    @property
    def admin_only_fields(self):
        return []

    def get_document_id_field(self):
        return 'uuid'

    @property
    def facets_with_options(self):
        return 'pxe_enabled',

    @property
    def resource_allowed_policy_target(self):
        return 'baremetal:port:get'

    @property
    def service_type(self):
        return 'baremetal'

    def _get_rbac_field_filters(self, request_context):
        return []

    def get_parent_id_field(self):
        return 'node_uuid'

    def get_objects(self):
        """Generator that lists all ports."""
        ironic_client = openstack_clients.get_ironicclient()
        for port in ironic_client.port.list(detail=True, limit=0):
            yield port

    def serialize(self, port):
        port_dict = port.to_dict()
        return serialize_resource(port_dict, resources.PORT_FIELDS)



# Copyright 2018 Verizon Wireless
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import gettext

from babel import localedata
import mock
import webob

from searchlight.common import wsgi
from searchlight import i18n
from searchlight.tests import utils as test_utils


class RequestTest(test_utils.BaseTestCase):

    def _set_expected_languages(self, all_locales=None, avail_locales=None):
        if all_locales is None:
            all_locales = []

        # Override localedata.locale_identifiers to return some locales.
        def returns_some_locales(*args, **kwargs):
            return all_locales

        self.stubs.Set(localedata, 'locale_identifiers', returns_some_locales)

        # Override gettext.find to return other than None for some languages.
        def fake_gettext_find(lang_id, *args, **kwargs):
            found_ret = '/glance/%s/LC_MESSAGES/glance.mo' % lang_id
            if avail_locales is None:
                # All locales are available.
                return found_ret
            languages = kwargs['languages']
            if languages[0] in avail_locales:
                return found_ret
            return None

        self.stubs.Set(gettext, 'find', fake_gettext_find)

    def test_language_accept_default(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Accept-Language"] = "zz-ZZ,zz;q=0.8"
        result = request.best_match_language()
        self.assertIsNone(result)

    def test_language_accept_none(self):
        request = wsgi.Request.blank('/tests/123')
        result = request.best_match_language()
        self.assertIsNone(result)

    def test_best_match_language_expected(self):
        # If Accept-Language is a supported language, best_match_language()
        # returns it.
        self._set_expected_languages(all_locales=['it'])

        req = wsgi.Request.blank('/', headers={'Accept-Language': 'it'})
        self.assertEqual('it', req.best_match_language())

    def test_request_match_language_unexpected(self):
        # If Accept-Language is a language we do not support,
        # best_match_language() returns None.
        self._set_expected_languages(all_locales=['it'])

        req = wsgi.Request.blank('/', headers={'Accept-Language': 'unknown'})
        self.assertIsNone(req.best_match_language())

    def test_best_match_language_unknown(self):
        # Test that we are actually invoking language negotiation by webob
        request = wsgi.Request.blank('/')
        accepted = 'unknown-lang'
        request.headers = {'Accept-Language': accepted}

        # TODO(rosmaita): simplify when lower_constraints has webob >= 1.8.1
        try:
            from webob.acceptparse import AcceptLanguageValidHeader  # noqa
            cls = webob.acceptparse.AcceptLanguageValidHeader
            funcname = 'lookup'
            # Bug #1765748: see comment in code in the function under test
            # to understand why this is the correct return value for the
            # webob 1.8.x mock
            retval = 'fake_LANG'
        except ImportError:
            cls = webob.acceptparse.AcceptLanguage
            funcname = 'best_match'
            retval = None

        with mock.patch.object(cls, funcname) as mocked_function:
            mocked_function.return_value = retval

            self.assertIsNone(request.best_match_language())
            mocked_function.assert_called_once()

        # If Accept-Language is missing or empty, match should be None
        request.headers = {'Accept-Language': ''}
        self.assertIsNone(request.best_match_language())
        request.headers.pop('Accept-Language')
        self.assertIsNone(request.best_match_language())


class ResourceTest(test_utils.BaseTestCase):

    @mock.patch.object(wsgi, 'translate_exception')
    def test_resource_call_error_handle_localized(self,
                                                  mock_translate_exception):
        class Controller(object):
            def delete(self, req, identity):
                raise webob.exc.HTTPBadRequest(explanation='Not Found')

        actions = {'action': 'delete', 'identity': 12}
        env = {'wsgiorg.routing_args': [None, actions]}
        request = wsgi.Request.blank('/tests/123', environ=env)
        message_es = 'No Encontrado'

        resource = wsgi.Resource(Controller(),
                                 wsgi.JSONRequestDeserializer(),
                                 None)
        translated_exc = webob.exc.HTTPBadRequest(message_es)
        mock_translate_exception.return_value = translated_exc

        e = self.assertRaises(webob.exc.HTTPBadRequest,
                              resource, request)
        self.assertEqual(message_es, str(e))

    @mock.patch.object(i18n, 'translate')
    def test_translate_exception(self, mock_translate):
        # TODO(rosmaita): simplify when lower_constraints has webob >= 1.8.1
        try:
            from webob.acceptparse import AcceptLanguageValidHeader  # noqa
            cls = webob.acceptparse.AcceptLanguageValidHeader
            funcname = 'lookup'
        except ImportError:
            cls = webob.acceptparse.AcceptLanguage
            funcname = 'best_match'

        with mock.patch.object(cls, funcname) as mocked_function:
            mock_translate.return_value = 'No Encontrado'
            mocked_function.return_value = 'de'

            req = wsgi.Request.blank('/tests/123')
            req.headers["Accept-Language"] = "de"

            e = webob.exc.HTTPNotFound(explanation='Not Found')
            e = wsgi.translate_exception(req, e)
            self.assertEqual('No Encontrado', e.explanation)



# Copyright 2015 Hewlett-Packard Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from elasticsearch import exceptions as es_exc
import mock

from oslo_config import cfg
import oslo_utils
import re
from searchlight.elasticsearch.plugins.base import resource_group_reg
from searchlight.elasticsearch.plugins import utils as plugin_utils
from searchlight.tests.unit import utils as unit_test_utils
from searchlight.tests import utils as test_utils

CONF = cfg.CONF
now = oslo_utils.timeutils.utcnow()
now_str = now.strftime(plugin_utils.FORMAT)


class TestPluginUtils(test_utils.BaseTestCase):
    def test_facet_value_query(self):
        fields = ['simple', 'nested.single', 'nested.list']
        field_types = ['string', 'nested', 'object']
        aggregation_query = plugin_utils.get_facets_query(fields, field_types,
                                                          10)

        expected = dict((
            unit_test_utils.simple_facet_field_agg('simple', size=10),
            unit_test_utils.complex_facet_field_agg('nested.single', size=10),
            unit_test_utils.complex_facet_field_agg('nested.list', size=10)
        ))
        self.assertEqual(expected, aggregation_query)

    def test_facet_result_transform(self):
        agg_results = {
            "simple": {
                "doc_count_error_upper_bound": 0,
                "sum_other_doc_count": 0,
                "buckets": [{"key": "VALUE1", "doc_count": 1}]
            },
            "nested.single": {
                "doc_count_error_upper_bound": 0,
                "sum_other_doc_count": 0,
                "nested.single": {
                    "buckets": [{
                        "key": "SINGLE_VALUE1",
                        "doc_count": 1,
                        "nested.single__unique_docs": {
                            "doc_count": 1
                        }
                    }]
                }
            },
            "nested.list": {
                "doc_count_error_upper_bound": 0,
                "sum_other_doc_count": 0,
                "nested.list": {
                    "doc_count_error_upper_bound": 0,
                    "sum_other_doc_count": 0,
                    "buckets": [{
                        "key": "LIST_VALUE1",
                        "doc_count": 2,
                        "nested.list__unique_docs": {
                            "doc_count": 1
                        }
                    }]
                }
            }
        }

        formatted = plugin_utils.transform_facets_results(
            agg_results,
            resource_type="fake")

        expected = {
            "simple": [
                {"key": "VALUE1", "doc_count": 1}
            ],
            "nested.single": [
                {"key": "SINGLE_VALUE1", "doc_count": 1}
            ],
            "nested.list": [
                {"key": "LIST_VALUE1", "doc_count": 1}
            ]
        }

        self.assertEqual(expected, formatted)

    def test_timestamp_conversion(self):
        timestamp = '2016-02-17 18:48:01.252228'
        converted = plugin_utils.timestamp_to_isotime(timestamp)
        self.assertEqual('2016-02-17T18:48:01Z', converted)

    def test_get_refresh_interval(self):
        mock_engine = mock.Mock()
        with mock.patch('searchlight.elasticsearch.get_api') as mock_api:
            mock_api.return_value = mock_engine

            # Test #1: no refresh_interval is set, return default interval
            mock_engine.indices.get_settings.return_value = {}
            interval = plugin_utils.get_index_refresh_interval('test-index')
            mock_engine.indices.get_settings.assert_called_with(
                'test-index',
                'index.refresh_interval'
            )
            self.assertEqual('1s', interval)

            # Test #2: get refresh_interval already set
            mock_engine.reset_mock()
            body = {
                'text-index': {
                    'settings': {
                        'index': {
                            'refresh_interval': '2s'}}}}
            mock_engine.indices.get_settings.return_value = body
            interval = plugin_utils.get_index_refresh_interval('text-index')
            self.assertEqual('2s', interval)

    def test_set_refresh_interval(self):
        mock_engine = mock.Mock()
        with mock.patch('searchlight.elasticsearch.get_api') as mock_api:
            mock_api.return_value = mock_engine
            plugin_utils.set_index_refresh_interval('test-index', '10s')
            expected_body = {
                'index': {
                    'refresh_interval': '10s'
                }
            }
            mock_engine.indices.put_settings.assert_called_with(expected_body,
                                                                'test-index')

    @mock.patch('searchlight.elasticsearch.get_api')
    def test_index_settings(self, mock_api):
        mock_engine = mock.Mock()
        mock_api.return_value = mock_engine

        with mock.patch.object(CONF, 'elasticsearch') as mock_settings:
            mock_settings.index_gc_deletes = '100s'
            mock_settings.index_settings = {
                'key1': 'value1',
                'index.key2': 'value2',
                'index.something.key3': 'value3'
            }

            with mock.patch('oslo_utils.timeutils.utcnow', return_value=now):
                plugin_utils.create_new_index('test')

        expected = {
            'index': {
                'key1': 'value1',
                'key2': 'value2',
                'something.key3': 'value3',
                'gc_deletes': '100s'
            }
        }
        mock_engine.indices.create.assert_called_with(index='test-' + now_str,
                                                      body=expected)

    @mock.patch('searchlight.elasticsearch.get_api')
    def test_no_index_settings(self, mock_api):
        mock_engine = mock.Mock()
        mock_api.return_value = mock_engine

        with mock.patch('searchlight.elasticsearch.plugins.'
                        'utils._get_index_settings_from_config',
                        return_value={}):
            with mock.patch('oslo_utils.timeutils.utcnow', return_value=now):
                plugin_utils.create_new_index('test')

        mock_engine.indices.create.assert_called_with(index='test-' + now_str)

    def test_verify_index_name(self):
        """Test resource group name configuration.
           Group name must only contain lowercase alphanumeric characters
           and underscores. The first character cannot be an underscore.
        """

        invalid_names = [
            '_abc',
            '__abc',
            'abc*',
            'abc!',
            'Abc'
        ]
        for name in invalid_names:
            self.assertEqual(
                None,
                re.match(resource_group_reg, name))

        valid_names = [
            'abc',
            'abc_'
            'a'
        ]
        for name in valid_names:
            self.assertIsNotNone(re.match(resource_group_reg, name))

    def test_find_missing_types(self):
        with mock.patch((
                'searchlight.elasticsearch.plugins.utils.searchlight'
                '.elasticsearch.get_api')) as mock_api:

            mock_engine = mock.Mock()
            mock_api.return_value = mock_engine
            # Test no mapping exists
            mock_engine.indices.get_mapping.return_value = {}
            results = plugin_utils.find_missing_types(
                {
                    'index': ['OS::Nova::Server',
                              'OS::Neutron::Subnet']
                }
            )
            mock_engine.indices.get_mapping.assert_called()
            self.assertEqual(
                (set([]), set(['OS::Nova::Server', 'OS::Neutron::Subnet'])),
                results
            )

            mock_engine.reset_mock()
            # Test no index exists
            mock_engine.indices.get_mapping.side_effect = \
                es_exc.NotFoundError()
            results = plugin_utils.find_missing_types(
                {
                    'searchlight-search': ['OS::Nova::Server']
                }
            )
            mock_engine.indices.get_mapping.assert_called()
            self.assertEqual(
                (set(['searchlight-search']), set([])),
                results)



#    Copyright (c) 2016 Kylin Cloud
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import mock

import novaclient.v2.hypervisors as novaclient_hypervisors

from searchlight.elasticsearch.plugins.nova import\
    hypervisors as hypervisors_plugin
import searchlight.tests.utils as test_utils


_now = datetime.datetime.utcnow()
updated_now = _now.strftime('%Y-%m-%dT%H:%M:%SZ')


def _hypervisor_fixture(hypervisor_id, **kwargs):
    attrs = {
        'status': 'enabled',
        'service': {
            'host': 'test',
            'disabled_reason': None,
            'id': 7
        },
        'vcpus_used': 20,
        'hypervisor_type': 'fake',
        'free_ram_mb': 799288,
        'local_gb_used': 0,
        'host_ip': '127.0.0.1',
        'id': hypervisor_id,
        'memory_mb': 800000,
        'current_workload': 0,
        'vcpus': 1000,
        'state': 'up',
        'running_vms': 20,
        'free_disk_gb': 600000,
        'hypervisor_version': 1000,
        'disk_available_least': 0,
        'local_gb': 600000,
        'cpu_info': '{"arch": "x86_64", "model": "Nehalem",'
                    '"vendor": "Intel", "features": ["pge", "clflush"],'
                    '"topology": {"cores": 1, "threads": 1, "sockets": 4}}',
        'memory_mb_used': 712
    }

    attrs.update(kwargs)
    hypervisor = mock.Mock(spec=novaclient_hypervisors.Hypervisor, **attrs)
    hypervisor.to_dict.return_value = attrs
    return hypervisor


class TestHypervisorLoaderPlugin(test_utils.BaseTestCase):
    def setUp(self):
        super(TestHypervisorLoaderPlugin, self).setUp()
        self.plugin = hypervisors_plugin.HypervisorIndex()
        self._create_fixtures()

    def _create_fixtures(self):
        self.hypervisor1 = _hypervisor_fixture(
            '1', hypervisor_hostname="host1")

    def test_index_name(self):
        self.assertEqual('searchlight', self.plugin.resource_group_name)

    def test_document_type(self):
        self.assertEqual('OS::Nova::Hypervisor',
                         self.plugin.get_document_type())

    def test_serialize(self):
        expected = {
            'cpu_info': {
                u'arch': u'x86_64',
                u'features': [u'pge', u'clflush'],
                u'model': u'Nehalem',
                u'topology': {u'cores': 1, u'sockets': 4, u'threads': 1},
                u'vendor': u'Intel'},
            'disk_available_least': 0,
            'host_ip': '127.0.0.1',
            'hypervisor_hostname': 'host1',
            'hypervisor_type': 'fake',
            'hypervisor_version': 1000,
            'id': '1',
            'local_gb': 600000,
            'memory_mb': 800000,
            'service': {'disabled_reason': None, 'host': 'test', 'id': 7},
            'state': 'up',
            'status': 'enabled',
            'updated_at': updated_now,
            'vcpus': 1000,
        }
        with mock.patch('searchlight.elasticsearch.plugins.utils.get_now_str',
                        return_value=updated_now):
            serialized = self.plugin.serialize(self.hypervisor1)
        self.assertEqual(expected, serialized)



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import cinderclient.exceptions
from oslo_log import log as logging

from searchlight.elasticsearch.plugins import base
from searchlight.elasticsearch.plugins.cinder import serialize_cinder_volume
from searchlight import pipeline


LOG = logging.getLogger(__name__)


class VolumeHandler(base.NotificationBase):
    """Handles cinder volume notifications. These can come as a result of
    a user action (like a create, delete, metadata edit etc) or as a result of
    periodic auditing notifications cinder sends
    """

    @classmethod
    def _get_notification_exchanges(cls):
        # Unlike most services cinder doesn't override the exchange name from
        # the oslo.messaging default
        return ['openstack']

    def get_event_handlers(self):
        return {
            'volume.update.end': self.create_or_update,
            'volume.create.end': self.create_or_update,
            'volume.delete.end': self.delete,
            # TODO(sjmc7) These could be implemented as scripted updates
            'volume.attach.end': self.create_or_update,
            'volume.detach.end': self.create_or_update,
            'volume.retype': self.create_or_update,
        }

    def get_log_fields(self, event_type, payload):
        return ('id', payload.get('volume_id')),

    def create_or_update(self, event_type, payload, timestamp):
        volume_id = payload['volume_id']
        LOG.debug("Updating cinder volume information for %s", volume_id)

        try:
            volume_payload = serialize_cinder_volume(volume_id)
            version = self.get_version(volume_payload, timestamp)
            self.index_helper.save_document(volume_payload, version=version)
            return pipeline.IndexItem(
                self.index_helper.plugin,
                event_type,
                payload,
                volume_payload
            )
        except cinderclient.exceptions.NotFound:
            LOG.warning("Volume %s not found; deleting" % volume_id)
            self.delete(payload, timestamp)

    def delete(self, event_type, payload, timestamp):
        volume_id = payload['volume_id']
        LOG.debug("Deleting cinder volume information for %s", volume_id)
        if not volume_id:
            return

        try:
            self.index_helper.delete_document({'_id': volume_id})
            return pipeline.DeleteItem(
                self.index_helper.plugin,
                event_type,
                payload,
                volume_id
            )
        except Exception as exc:
            LOG.error(
                'Error deleting volume %(volume_id)s '
                'from index. Error: %(exc)s' %
                {'volume_id': volume_id, 'exc': exc})



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import mock

from searchlight.elasticsearch.plugins import base

ROLE_SEPARATED_DATA = [
    {
        "id": "role-fake1",
        "public_field": "this is public",
        "admin_wildcard_this": "this is admin only",
        "admin_wildcard_that": "this is admin too",
        "admin_specific": "specific admin field",
        "tenant_id": "tenant1",
        "updated_at": "2015-08-06T12:48:14.000000"
    },
    {
        "id": "role-fake2",
        "public_field": "still public",
        "admin_wildcard_this": "still admin",
        "admin_wildcard_that": "still admin too",
        "admin_specific": "specific admin field",
        "tenant_id": "tenant1",
        "updated_at": "2015-08-06T12:48:14.000000"
    }
]


NON_ROLE_SEPARATED_DATA = [
    {
        "id": "non-role-fake1",
        "public_field": "this is public",
        "tenant_id": "tenant1",
        "updated_at": "2015-08-06T12:48:14.000000"
    },
    {
        "id": "non-role-fake2",
        "public_field": "still public",
        "tenant_id": "tenant1",
        "updated_at": "2015-08-06T12:48:14.000000"
    }
]

ROUTING_DATA = [
    {
        "id": "id_for_routing_plugin-fake1",
        "tenant_id": "tenant1",
    }
]

SIMPLE_DATA = [
    {
        "id": "simple1",
        "updated_at": "2015-08-06T12:48:14.000000"
    }
]


CHILD_DATA = [
    {
        "id": "child1",
        "parent_id": "simple1",
        "updated_at": "2015-08-06T12:48:14.000000"
    }
]


class FakePluginBase(base.IndexBase):
    NotificationHandlerCls = base.NotificationBase

    def __init__(self, es_engine):
        self.options = mock.Mock()
        self.options.admin_only_fields = None
        self.options.resource_group_name = 'searchlight'
        self.options.enabled = True
        self.options.mapping_use_doc_values = True
        self.options.override_region_name = None

        self.engine = es_engine
        self.document_type = self.get_document_type()
        self.parent_plugin = None
        self.child_plugins = []

    def serialize(self, doc):
        return doc

    def _get_rbac_field_filters(self, request_context):
        return []

    @property
    def resource_allowed_policy_target(self):
        return None

    @property
    def service_type(self):
        return None


class RoleSeparatedPlugin(FakePluginBase):
    def __init__(self, es_engine):
        super(RoleSeparatedPlugin, self).__init__(es_engine)
        self.options.admin_only_fields = 'admin_wildcard_*,admin_specific'

    @classmethod
    def get_document_type(cls):
        return 'role-separated'

    def get_mapping(self):
        return {
            'properties': {
                'id': {'type': 'string', 'index': 'not_analyzed'},
                'public_field': {'type': 'string'},
                'admin_wildcard_this': {'type': 'string'},
                'admin_wildcard_that': {'type': 'string'},
                'admin_specific': {'type': 'string'},
                'tenant_id': {'type': 'string'}
            }
        }

    def _get_rbac_field_filters(self, request_context):
        return [
            {'term': {'tenant_id': request_context.owner}}
        ]

    def get_objects(self):
        self.number_documents = len(ROLE_SEPARATED_DATA)
        return copy.deepcopy(ROLE_SEPARATED_DATA)

    def serialize(self, doc):
        return doc


class NonRoleSeparatedPlugin(FakePluginBase):
    def __init__(self, es_engine):
        super(NonRoleSeparatedPlugin, self).__init__(es_engine)

    @classmethod
    def get_document_type(cls):
        return 'not-role-separated'

    def get_mapping(self):
        return {
            'properties': {
                'id': {'type': 'string', 'index': 'not_analyzed'},
                'public_field': {'type': 'string'},
                'tenant_id': {'type': 'string'},
                'faceted': {'type': 'short'}
            }
        }

    def _get_rbac_field_filters(self, request_context):
        return [
            {'term': {'tenant_id': request_context.owner}}
        ]

    @property
    def facets_with_options(self):
        return ('faceted',)

    def get_objects(self):
        self.number_documents = len(NON_ROLE_SEPARATED_DATA)
        return copy.deepcopy(NON_ROLE_SEPARATED_DATA)

    def serialize(self, doc):
        return doc


class FakeSimpleRoutingPlugin(FakePluginBase):
    def __init__(self, es_engine):
        super(FakeSimpleRoutingPlugin, self).__init__(es_engine)

    @classmethod
    def get_document_type(cls):
        return 'Fake-RoutingPlugin'

    def get_mapping(self):
        return {
            'properties': {
                'id': {'type': 'string', 'index': 'not_analyzed'},
                'tenant_id': {'type': 'string'}
            }
        }

    def _get_rbac_field_filters(self, request_context):
        return [
            {'term': {'tenant_id': request_context.owner}}
        ]

    @property
    def routing_field(self):
        return "tenant_id"

    def get_objects(self):
        self.number_documents = len(ROUTING_DATA)
        return copy.deepcopy(ROUTING_DATA)

    def serialize(self, doc):
        return doc


class FakeSimplePlugin(FakePluginBase):
    def __init__(self, es_engine):
        super(FakeSimplePlugin, self).__init__(es_engine)

    @classmethod
    def get_document_type(cls):
        return 'fake-simple'

    def get_mapping(self):
        return {
            'properties': {
                'id': {'type': 'string', 'index': 'not_analyzed'}
            }
        }

    def get_objects(self):
        self.number_documents = len(SIMPLE_DATA)
        return copy.deepcopy(SIMPLE_DATA)


class FakeChildPlugin(FakePluginBase):
    def __init__(self, es_engine):
        super(FakeChildPlugin, self).__init__(es_engine)

    @classmethod
    def get_document_type(cls):
        return 'fake-child'

    def get_mapping(self):
        # Explicit parent is not necessary; it'll get added
        return {
            'properties': {
                'id': {'type': 'string', 'index': 'not_analyzed'},
                'parent_id': {'type': 'string', 'index': 'not_analyzed'}
            }
        }

    @classmethod
    def parent_plugin_type(cls):
        return "fake-simple"

    def get_parent_id_field(self):
        return 'parent_id'

    def get_objects(self):
        self.number_documents = len(CHILD_DATA)
        return copy.deepcopy(CHILD_DATA)


class FakeWrongGroupChildPlugin(FakeChildPlugin):
    def __init__(self, es_engine):
        super(FakeWrongGroupChildPlugin, self).__init__(es_engine)
        self.options.resource_group_name = 'wrong-group-name'

    @classmethod
    def get_document_type(cls):
        return 'fake-wrong-index-child'


class FakeWrongGroupGrandchildPlugin(FakeChildPlugin):
    def __init__(self, es_engine):
        super(FakeWrongGroupGrandchildPlugin, self).__init__(es_engine)
        self.options.resource_group_name = 'wrong-group-name'

    @classmethod
    def get_document_type(cls):
        return 'fake-wrong-index-grand-child'

    @classmethod
    def parent_plugin_type(cls):
        return "fake-wrong-index-child"


class FakeSeparatedChildPlugin(FakePluginBase):
    def __init__(self, es_engine):
        super(FakeSeparatedChildPlugin, self).__init__(es_engine)

    @classmethod
    def get_document_type(cls):
        return 'fake-role-separated-child'

    def get_mapping(self):
        # Explicit parent is not necessary; it'll get added
        return {
            'properties': {
                'id': {'type': 'string', 'index': 'not_analyzed'},
                'parent_id': {'type': 'string', 'index': 'not_analyzed'}
            }
        }

    @classmethod
    def parent_plugin_type(cls):
        return "role-separated"

    def get_parent_id_field(self):
        return 'parent_id'

    @property
    def requires_role_separation(self):
        return True

    def get_objects(self):
        self.number_documents = len(CHILD_DATA)
        return copy.deepcopy(CHILD_DATA)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from searchlight.common import resource_types
from searchlight.elasticsearch.plugins import base
from searchlight.elasticsearch.plugins.ironic import notification_handlers
from searchlight.elasticsearch.plugins.ironic import resources
from searchlight.elasticsearch.plugins.ironic import serialize_resource
from searchlight.elasticsearch.plugins import openstack_clients


# chassis is not mandatory for node, and it is not parent plugin
class ChassisIndex(base.IndexBase):
    NotificationHandlerCls = notification_handlers.ChassisHandler

    @classmethod
    def get_document_type(cls):
        return resource_types.IRONIC_CHASSIS

    def get_mapping(self):
        return copy.deepcopy(resources.CHASSIS_MAPPING)

    @property
    def admin_only_fields(self):
        return []

    def get_document_id_field(self):
        return 'uuid'

    @property
    def resource_allowed_policy_target(self):
        return 'baremetal:chassis:get'

    @property
    def service_type(self):
        return 'baremetal'

    def _get_rbac_field_filters(self, request_context):
        return []

    def get_objects(self):
        """Generator that lists all chassis."""
        ironic_client = openstack_clients.get_ironicclient()
        for chassis in ironic_client.chassis.list(detail=True, limit=0):
            yield chassis

    def serialize(self, chassis):
        chassis_dict = chassis.to_dict()
        return serialize_resource(chassis_dict, resources.CHASSIS_FIELDS)



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from searchlight.common import resource_types
from searchlight.elasticsearch.plugins import base
from searchlight.elasticsearch.plugins.cinder import serialize_cinder_snapshot
from searchlight.elasticsearch.plugins.cinder import \
    snapshots_notification_handler
from searchlight.elasticsearch.plugins import openstack_clients

LOG = logging.getLogger(__name__)


# TODO(sjmc7): Parameterize once we have plugin configs
LIST_LIMIT = 100


class SnapshotIndex(base.IndexBase):
    NotificationHandlerCls = snapshots_notification_handler.SnapshotHandler

    ADMIN_ONLY_FIELDS = []

    @classmethod
    def get_document_type(cls):
        """Unusually, this doesn't have a heat resource type equivalent"""
        return resource_types.CINDER_SNAPSHOT

    @classmethod
    def parent_plugin_type(cls):
        return resource_types.CINDER_VOLUME

    def get_parent_id_field(self):
        return 'volume_id'

    @property
    def resource_allowed_policy_target(self):
        return 'volume:get_all_snapshots'

    @property
    def service_type(self):
        return 'volume'

    @property
    def requires_role_separation(self):
        # TODO(sjmc7) Remove once this is abstracted
        return self.parent_plugin.requires_role_separation

    def get_mapping(self):
        return {
            'dynamic': True,
            'properties': {
                'id': {'type': 'string', 'index': 'not_analyzed'},
                'description': {'type': 'string'},
                'name': {
                    'type': 'string',
                    'fields': {
                        'raw': {'type': 'string', 'index': 'not_analyzed'}
                    }
                },
                'os-extended-snapshot-attributes:progress':
                    {'type': 'string', 'index': 'not_analyzed'},
                'os-extended-snapshot-attributes:project_id':
                    {'type': 'string', 'index': 'not_analyzed'},
                'user_id': {'type': 'string', 'index': 'not_analyzed'},
                'created_at': {'type': 'date'},
                'updated_at': {'type': 'date'},
                'status': {'type': 'string', 'index': 'not_analyzed'},
                'tenant_id': {'type': 'string', 'index': 'not_analyzed'},
                'project_id': {'type': 'string', 'index': 'not_analyzed'},
                'volume_id': {'type': 'string', 'index': 'not_analyzed'},
            },
            "_meta": {
                "tenant_id": {
                    "resource_type": resource_types.KEYSTONE_PROJECT
                },
                "project_id": {
                    "resource_type": resource_types.KEYSTONE_PROJECT
                },
                "os-extended-snapshot-attributes:project_id": {
                    "resource_type": resource_types.KEYSTONE_PROJECT
                },
                "user_id": {
                    "resource_type": resource_types.KEYSTONE_USER
                }
            }
        }

    @property
    def admin_only_fields(self):
        from_conf = super(SnapshotIndex, self).admin_only_fields
        return SnapshotIndex.ADMIN_ONLY_FIELDS + from_conf

    @property
    def facets_with_options(self):
        return ('status',)

    @property
    def facets_excluded(self):
        """A map of {name: allow_admin} that indicate which
        fields should not be offered as facet options, or those that should
        only be available to administrators.
        """
        # FIXME: DuncanT: Need to go through these carefully
        return {'user_id': True,
                'tenant_id': True,
                'os-extended-snapshot-attributes:project_id': False}

    def _get_rbac_field_filters(self, request_context):
        """Return any RBAC field filters to be injected into an indices
        query. Document type will be added to this list.
        """
        return [
            {'term': {'project_id': request_context.owner}}
        ]

    def get_objects(self):
        """Generator that lists all cinder snapshots owned by all tenants."""
        LOG.debug("Cinder snapshots get_objects started")
        has_more = True
        marker = None
        cc = openstack_clients.get_cinderclient()
        while has_more:
            snapshots = cc.volume_snapshots.list(
                limit=LIST_LIMIT,
                search_opts={'all_tenants': True},
                marker=marker
            )

            if not snapshots:
                # Definitely no more; break straight away
                break

            # snapshots.list always returns a list so we can grab the last id
            has_more = len(snapshots) == LIST_LIMIT
            marker = snapshots[-1].id

            for snapshot in snapshots:
                yield snapshot

    def serialize(self, snapshot):
        return serialize_cinder_snapshot(snapshot)



#!/usr/bin/env python

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2011 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Searchlight API Server
"""

import eventlet
import os
import sys

from oslo_config import cfg
from oslo_log import log as logging
from oslo_reports import guru_meditation_report as gmr
from oslo_utils import encodeutils
import osprofiler.notifier
import osprofiler.web

from searchlight.common import config
from searchlight.common import exception
from searchlight.common import utils
from searchlight.common import wsgi
from searchlight import notifier
from searchlight import service_policies
from searchlight import version

# Monkey patch socket, time, select, threads
# NOTE(sjmc7): to workaround issue with 2.7.12-1ubuntu0~16.04.3 and
# eventlet < 0.22.0 we need to sleep before monkey patching.
# For details please check https://bugs.launchpad.net/neutron/+bug/1745013
# See https://review.opendev.org/#/c/537863
eventlet.sleep()
eventlet.patcher.monkey_patch(socket=True, time=True, select=True,
                              thread=True, os=True)

# If ../searchlight/__init__.py exists, add ../ to Python search path, so that
# it will override what happens to be installed in /usr/(local/)lib/python...
possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                   os.pardir,
                                   os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'searchlight', '__init__.py')):
    sys.path.insert(0, possible_topdir)

CONF = cfg.CONF
CONF.import_group("profiler", "searchlight.common.wsgi")
CONF.import_group("api", "searchlight.common.wsgi")
logging.register_options(CONF)

KNOWN_EXCEPTIONS = (RuntimeError,
                    exception.WorkerCreationFailure)


def fail(e):
    global KNOWN_EXCEPTIONS
    return_code = KNOWN_EXCEPTIONS.index(type(e)) + 1
    sys.stderr.write("ERROR: %s\n" % encodeutils.exception_to_unicode(e))
    sys.exit(return_code)


def configure_wsgi():
    config.parse_args()
    config.set_config_defaults()
    logging.setup(CONF, 'searchlight')
    gmr.TextGuruMeditation.setup_autorun(version)
    utils.register_plugin_opts()

    # Fail fast if service policy files aren't found
    service_policies.check_policy_files()

    if CONF.profiler.enabled:
        _notifier = osprofiler.notifier.create("Messaging",
                                               notifier.messaging, {},
                                               notifier.get_transport(),
                                               "searchlight", "search",
                                               CONF.api.bind_host)
        osprofiler.notifier.set(_notifier)
    else:
        osprofiler.web.disable()


def main():
    try:
        configure_wsgi()
        wsgi.set_eventlet_hub()
        server = wsgi.Server(workers=CONF.api.workers)
        server.start(config.load_paste_app('searchlight'),
                     default_port=9393)
        server.wait()
    except KNOWN_EXCEPTIONS as e:
        fail(e)


if __name__ == '__main__':
    main()



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from searchlight.common import resource_types
from searchlight.elasticsearch.plugins import designate
from searchlight.elasticsearch.plugins.designate import notification_handlers


class ZoneIndex(designate.DesignateBase):
    NotificationHandlerCls = notification_handlers.ZoneHandler

    def get_notification_handler(self):
        """Override because the zone handler needs a handle to recordset
        indexer (for initial recordset indexing).
        """
        return self.NotificationHandlerCls(
            self.index_helper,
            self.options,
            recordset_helper=self.child_plugins[0].index_helper)

    @classmethod
    def get_document_type(cls):
        return resource_types.DESIGNATE_ZONE

    def get_mapping(self):
        return {
            "dynamic": True,
            "dynamic_templates": [
                {
                    "_id": {
                        "match": "*_id",
                        "match_mapping_type": "string",
                        "mapping": {
                            "type": "string",
                            "index": "not_analyzed"
                        }
                    }
                }
            ],
            "properties": {
                "id": {"type": "string", "index": "not_analyzed"},
                "project_id": {"type": "string", "index": "not_analyzed"},
                "created_at": {"type": "date"},
                "updated_at": {"type": "date"},
                "name": {
                    "type": "string",
                    "fields": {
                        "raw": {"type": "string", "index": "not_analyzed"}
                    }
                },
                "description": {"type": "string"},
                "version": {"type": "integer"},
                "shard": {"type": "integer"},
                "email": {"type": "string"},
                "ttl": {"type": "integer"},
                "refresh": {"type": "integer"},
                "retry": {"type": "integer"},
                "expire": {"type": "integer"},
                "minimum": {"type": "integer"},
                "serial": {"type": "integer"},
                "status": {"type": "string", "index": "not_analyzed"},
                "action": {"type": "string", "index": "not_analyzed"},
                "type": {"type": "string", "index": "not_analyzed"},
                "transferred_at": {"type": "string"},
                "masters": {"type": "string"}
            },
            "_meta": {
                "project_id": {
                    "resource_type": resource_types.KEYSTONE_PROJECT
                }
            }
        }

    @property
    def facets_with_options(self):
        return ('status', 'type')

    @property
    def facets_excluded(self):
        """Facets either not available or available only to admins"""
        return {'project_id': True}

    @property
    def resource_allowed_policy_target(self):
        return 'get_zones'

    @property
    def service_type(self):
        return 'dns'

    def _get_rbac_field_filters(self, request_context):
        return [
            {"term": {"project_id": request_context.owner}}
        ]

    def get_objects(self):
        for zone in designate._get_zones():
            yield zone

    def serialize(self, obj):
        obj.pop("links", None)
        if not obj['updated_at'] and obj['created_at']:
            obj['updated_at'] = obj['created_at']
        if 'project_id' not in obj:
            obj['project_id'] = obj['tenant_id']
        return obj



# Copyright (c) 2015-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import mock

from oslo_serialization import jsonutils
from oslo_utils import uuidutils

from searchlight.listener import NotificationEndpoint
from searchlight.pipeline import PipelineManager
from searchlight.tests import functional
from searchlight.tests.functional import generate_load_data
from searchlight.tests.functional import mock_glance_pyclient
from searchlight.tests.functional import test_api
from searchlight.tests.functional import test_listener
from searchlight.tests import utils


member_list = 'glanceclient.v2.image_members.Controller.list'


class TestGlancePlugins(functional.FunctionalTest):
    def setUp(self):
        super(TestGlancePlugins, self).setUp()
        self.images_plugin = self.initialized_plugins['OS::Glance::Image']
        self.metadefs_plugin = self.initialized_plugins['OS::Glance::Metadef']

    def test_image_property_protection(self):
        doc_with_properties = {
            "owner": test_api.TENANT1,
            "id": uuidutils.generate_uuid(),
            "name": "doc with properties",
            "x_none_permitted": "nobody can do anything",
            "x_foo_matcher": "admin only",
            "x_owner_anything": "member or admin",
            "x_none_read": "nobody can read",
            "any_old_property": "restricted to admins",
            "x_foo_anybody": "anybody may read",
            "spl_read_only_prop": "spl_role only",
            "visibility": "public",
            "created_at": "2016-04-06T12:48:18Z"
        }
        self._index(self.images_plugin,
                    [doc_with_properties])

        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT1)
        self.assertEqual(200, response.status_code)
        # Image_type is metadata on image and not a 'core' glance property.
        # Property protection policy defines access similar to other metadata
        # here.
        # We need to remove 'image_type' since non-core properties are
        # allowed only for admin role defined in test policy file located at
        # searchlight/tests/etc/property-protections.conf (See last section)
        expect_removed = ["x_none_permitted", "x_foo_matcher", "x_none_read",
                          "any_old_property", "spl_read_only_prop",
                          'image_type']
        expected_result = dict((k, v)
                               for k, v in doc_with_properties.items()
                               if k not in expect_removed)
        expected_result['members'] = []
        expected_result['project_id'] = test_api.TENANT1
        self.assertEqual([expected_result], self._get_hit_source(json_content))

        # Test with the 'spl_role' role
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT1,
                                                      role="spl_role")
        self.assertEqual(200, response.status_code)
        expect_removed = ["x_none_permitted", "x_foo_matcher", "x_none_read",
                          "any_old_property", "x_owner_anything",
                          "image_type"]
        expected_result = dict((k, v)
                               for k, v in doc_with_properties.items()
                               if k not in expect_removed)
        expected_result['members'] = []
        expected_result['project_id'] = test_api.TENANT1

        self.assertEqual([expected_result], self._get_hit_source(json_content))

        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT1,
                                                      role="admin")
        self.assertEqual(200, response.status_code)
        expect_removed = ["x_none_permitted", "x_none_read"]
        expected_result = dict((k, v)
                               for k, v in doc_with_properties.items()
                               if k not in expect_removed)
        expected_result['members'] = []
        expected_result['project_id'] = test_api.TENANT1
        expected_result['image_type'] = 'image'

        self.assertEqual([expected_result], self._get_hit_source(json_content))

    def test_rbac_admin(self):
        """Test that an admin has access to everything"""
        image_doc = {
            "owner": test_api.TENANT1,
            "id": uuidutils.generate_uuid(),
            "name": "abc",
            "visibility": "private",
            "members": [],
            "created_at": "2016-04-06T12:48:18Z"
        }
        metadef_doc = {
            "owner": test_api.TENANT2,
            "visibility": "private",
            "namespace": "some.value1",
            "objects": [],
            "tags": [],
            "properties": {},
            "created_at": "2016-04-06T12:48:18Z"
        }

        fake_member = {"member_id": test_api.TENANT1, "status": "accepted"}
        with mock.patch(member_list, return_value=[fake_member]):
            self._index(self.images_plugin, [image_doc])

        self._index(self.metadefs_plugin, [metadef_doc])

        # Expected output format is a bit different
        metadef_doc.update({
            'properties': [],
            'protected': None,
            'resource_types': [],
            'name': metadef_doc['namespace'],
            'id': metadef_doc['namespace'],
            'description': None,
            'display_name': None,
            'updated_at': None,
            'project_id': test_api.TENANT2
        })
        image_doc['project_id'] = test_api.TENANT1

        # An ordinary user in TENANT3 shouldn"t have any access
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT3)
        self.assertEqual([], self._get_hit_source(json_content))

        # An admin without specifying all_projects should get the same
        # result as an ordinary user
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT3,
                                                      role='admin')
        self.assertEqual([], self._get_hit_source(json_content))

        # An admin should have access to all (at least in KS v2)
        admin_match_all = {
            'all_projects': True
        }
        admin_match_all.update(test_api.MATCH_ALL)
        response, json_content = self._search_request(admin_match_all,
                                                      test_api.TENANT3,
                                                      role='admin')
        image_doc['image_type'] = 'image'
        self.assertEqual([image_doc, metadef_doc],
                         self._get_hit_source(json_content))

    def test_image_rbac_owner(self):
        """Test glance.image RBAC based on the "owner" field"""
        id_1 = uuidutils.generate_uuid()
        tenant1_doc = {
            "owner": test_api.TENANT1,
            "id": id_1,
            "visibility": "private",
            "name": "owned by tenant 1",
            "created_at": "2016-04-06T12:48:18Z"
        }
        tenant2_doc = {
            "owner": test_api.TENANT2,
            "id": uuidutils.generate_uuid(),
            "visibility": "private",
            "name": "owned by tenant 2",
            "created_at": "2016-04-06T12:48:18Z"
        }

        with mock.patch(member_list, return_value=[]):
            self._index(self.images_plugin,
                        [tenant1_doc, tenant2_doc])

        tenant1_doc["members"] = []
        tenant2_doc["members"] = []
        tenant1_doc["project_id"] = tenant1_doc["owner"]
        tenant2_doc["project_id"] = tenant2_doc["owner"]

        # Query for everything as one tenant then the other
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT1)
        self.assertEqual(200, response.status_code)
        self.assertEqual([tenant1_doc], self._get_hit_source(json_content))

        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT2)
        self.assertEqual(200, response.status_code)
        self.assertEqual([tenant2_doc], self._get_hit_source(json_content))

        # Query the hidden doc explicitly
        query = {
            "query": {
                "match": {"id": id_1}
            }
        }
        response, json_content = self._search_request(query,
                                                      test_api.TENANT2)
        self.assertEqual([], self._get_hit_source(json_content))

    def test_image_rbac_member(self):
        """Test glance.image RBAC based on the "member" field"""
        owner = uuidutils.generate_uuid()
        accessible_doc = {
            "owner": owner,
            "id": uuidutils.generate_uuid(),
            "visibility": "shared",
            "name": "accessible doc",
            "created_at": "2016-04-06T12:48:18Z"
        }
        inaccessible_doc = {
            "owner": uuidutils.generate_uuid(),
            "id": uuidutils.generate_uuid(),
            "visibility": "shared",
            "name": "inaccessible_doc doc",
            "created_at": "2016-04-06T12:48:18Z"
        }

        # Assign TENANT1, TENANT2 to accessible doc and a fake member to
        # inaccessible doc
        made_up_tenant = uuidutils.generate_uuid()
        fake_members = (
            [{"member_id": test_api.TENANT1, "status": "accepted"},
             {"member_id": test_api.TENANT2, "status": "accepted"}],
            [{"member_id": made_up_tenant, "status": "accepted"}]
        )
        with mock.patch(member_list, side_effect=fake_members):
            self._index(self.images_plugin,
                        [accessible_doc, inaccessible_doc])

        accessible_doc["project_id"] = accessible_doc["owner"]
        inaccessible_doc["members"] = [made_up_tenant]

        # Someone in TENANT1 or TENANT2 should have access to "accessible_doc"
        # but should only see their tenant in 'members'
        accessible_doc["members"] = [test_api.TENANT1]
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT1)
        self.assertEqual(200, response.status_code)
        self.assertEqual([accessible_doc], self._get_hit_source(json_content))

        accessible_doc["members"] = [test_api.TENANT2]
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT2)
        self.assertEqual(200, response.status_code)
        self.assertEqual([accessible_doc], self._get_hit_source(json_content))

        # A user in 'owner' should see the member list
        accessible_doc["members"] = [test_api.TENANT1, test_api.TENANT2]
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      owner)
        self.assertEqual(200, response.status_code)
        self.assertEqual([accessible_doc], self._get_hit_source(json_content))

        # An admin should see the member list even if they're in another tenant
        accessible_doc["members"] = [test_api.TENANT1, test_api.TENANT2]
        accessible_doc["image_type"] = "image"
        response, json_content = self._search_request(
            {"query": {"term": {"owner": owner}}, "all_projects": True},
            uuidutils.generate_uuid(),
            role="admin")
        self.assertEqual(200, response.status_code)
        self.assertEqual([accessible_doc], self._get_hit_source(json_content))

        # A user in another tenant shouldn't see it at all
        response, json_content = self._search_request(
            test_api.MATCH_ALL, uuidutils.generate_uuid())
        self.assertEqual(200, response.status_code)
        self.assertEqual([], self._get_hit_source(json_content))

    def test_image_rbac_visibility(self):
        """Test that "visibility: public" makes images visible"""
        visible_doc = {
            "owner": uuidutils.generate_uuid(),
            "id": uuidutils.generate_uuid(),
            "visibility": "public",
            "name": "visible doc",
            "created_at": "2016-04-06T12:48:18Z"
        }
        invisible_doc = {
            "owner": uuidutils.generate_uuid(),
            "id": uuidutils.generate_uuid(),
            "visibility": "private",
            "name": "visible doc",
            "created_at": "2016-04-06T12:48:18Z"
        }

        # Generate a fake tenant id for invisible_doc's "members"
        fake_member = {"member_id": uuidutils.generate_uuid(),
                       "status": "accepted"}
        with mock.patch(member_list, return_value=[fake_member]):
            self._index(self.images_plugin,
                        [visible_doc, invisible_doc],
                        test_api.TENANT1)

        visible_doc["members"] = []
        visible_doc["project_id"] = visible_doc["owner"]

        # visible doc should be visible to any user
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT2)
        self.assertEqual(200, response.status_code)
        self.assertEqual([visible_doc], self._get_hit_source(json_content))

    def test_metadef_rbac_visibility(self):
        visible_doc = {
            "owner": uuidutils.generate_uuid(),
            "id": uuidutils.generate_uuid(),
            "visibility": "public",
            "name": "visible doc",
            "namespace": "TEST_VISIBLE",
            "properties": {},
            "tags": [],
            "objects": [],
            "created_at": "2016-04-06T12:48:18Z"
        }
        invisible_doc = {
            "owner": uuidutils.generate_uuid(),
            "id": uuidutils.generate_uuid(),
            "visibility": "private",
            "name": "visible doc",
            "namespace": "TEST_INVISIBLE",
            "properties": {},
            "tags": [],
            "objects": [],
            "created_at": "2016-04-06T12:48:18Z"
        }
        self._index(self.metadefs_plugin,
                    [visible_doc, invisible_doc])

        visible_doc["id"] = visible_doc["namespace"]
        visible_doc["name"] = visible_doc["namespace"]
        visible_doc["project_id"] = visible_doc["owner"]
        visible_doc.update({
            "resource_types": [],
            "properties": [],
            "description": None,
            "display_name": None,
            "protected": None,
            "updated_at": None
        })

        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT2)
        self.assertEqual(200, response.status_code)
        self.assertEqual([visible_doc], self._get_hit_source(json_content))

    def test_metadef_rbac_owner(self):
        visible_doc = {
            "owner": test_api.TENANT1,
            "namespace": "VISIBLE",
            "visibility": "private",
            "properties": {},
            "tags": [],
            "objects": [],
            "created_at": "2016-04-06T12:48:18Z"
        }
        invisible_doc = {
            "owner": uuidutils.generate_uuid(),
            "namespace": "INVISIBLE",
            "visibility": "private",
            "properties": {},
            "tags": [],
            "objects": [],
            "created_at": "2016-04-06T12:48:18Z"
        }
        self._index(self.metadefs_plugin,
                    [visible_doc, invisible_doc])

        visible_doc["id"] = visible_doc["namespace"]
        visible_doc["name"] = visible_doc["namespace"]
        visible_doc["project_id"] = visible_doc["owner"]
        visible_doc.update({
            "resource_types": [],
            "properties": [],
            "description": None,
            "display_name": None,
            "protected": None,
            "updated_at": None
        })

        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT1)
        self.assertEqual(200, response.status_code)
        self.assertEqual([visible_doc], self._get_hit_source(json_content))

        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT2)
        self.assertEqual(200, response.status_code)
        self.assertEqual([], self._get_hit_source(json_content))

    def test_index_with_dot_in_field(self):
        test_doc = {
            "owner": test_api.TENANT1,
            "id": uuidutils.generate_uuid(),
            "name": "test image",
            "visibility": "public",
            "property_1.2.3": "test_property",
            "created_at": "2016-04-06T12:48:18Z"
        }

        self._index(self.images_plugin, [test_doc])
        response, json_content = self._search_request(test_api.MATCH_ALL,
                                                      test_api.TENANT1,
                                                      role="admin")
        self.assertEqual(200, response.status_code)
        expected = test_doc.copy()
        expected.update(image_type="image", members=[],
                        project_id=test_api.TENANT1)
        self.assertEqual([expected], self._get_hit_source(json_content))


class TestGlanceListener(test_listener.TestSearchListenerBase):
    def __init__(self, *args, **kwargs):
        super(TestGlanceListener, self).__init__(*args, **kwargs)

        self.image_events = self._load_fixture_data('events/images.json')
        self.metadef_events = self._load_fixture_data('events/metadefs.json')

    def setUp(self):
        super(TestGlanceListener, self).setUp()
        openstack_client_mod = "searchlight.elasticsearch.plugins." \
                               "openstack_clients.get_glanceclient"
        osclient_patcher = mock.patch(
            openstack_client_mod, mock_glance_pyclient.get_fake_glance_client
        )
        osclient_patcher.start()
        self.addCleanup(osclient_patcher.stop)

        self.images_plugin = self.initialized_plugins['OS::Glance::Image']
        self.metadefs_plugin = self.initialized_plugins['OS::Glance::Metadef']

        notification_plugins = {
            plugin.document_type: utils.StevedoreMock(plugin)
            for plugin in (self.images_plugin, self.metadefs_plugin)}
        self.notification_endpoint = NotificationEndpoint(
            notification_plugins,
            PipelineManager(notification_plugins)
        )

        self.images_index = self.images_plugin.alias_name_listener
        self.metadefs_index = self.metadefs_plugin.alias_name_listener

    def test_image_create_event(self):
        """Send image.create notification event to listener"""
        create_event = self.image_events["image.create"]
        self._send_event_to_listener(create_event, self.images_index)
        result = self._verify_event_processing(create_event)
        verification_keys = ['id', 'status']
        self._verify_result(create_event, verification_keys, result)

    def test_image_update_conflict(self):
        """Send an outdated image.update notification event to listener,
           test if the document will be updated
        """
        create_event = self.image_events["image.create"]
        self._send_event_to_listener(create_event, self.images_index)

        update_event = self.image_events["image.update"]
        self._send_event_to_listener(update_event, self.images_index)

        outdate_update_event = copy.deepcopy(update_event)
        outdate_update_event['payload']['updated_at'] = ("2015-09-01T"
                                                         "09:06:18.000000")
        self._send_event_to_listener(outdate_update_event, self.images_index)
        result = self._verify_event_processing(update_event)

        # Check if outdated update notification failed
        verification_keys = ['updated_at']
        self._verify_result(update_event, verification_keys, result)

    def test_image_update_event(self):
        """Send image.update notification event to listener"""

        create_event = self.image_events["image.create"]
        self._send_event_to_listener(create_event, self.images_index)

        update_event = self.image_events["image.update"]
        self._send_event_to_listener(update_event, self.images_index)
        result = self._verify_event_processing(update_event)
        verification_keys = ['name', 'protected']
        self._verify_result(update_event, verification_keys, result)

    def test_image_delete_event(self):
        """Send image.delete notification event to listener"""

        create_event = self.image_events["image.create"]
        self._send_event_to_listener(create_event, self.images_index)
        self._verify_event_processing(create_event)

        delete_event = self.image_events["image.delete"]
        self._send_event_to_listener(delete_event, self.images_index)
        self._verify_event_processing(delete_event, 0)

    def test_image_member_create_event(self):
        """Send member.create notification event to listener"""

        create_event = self.image_events["image.create"]
        self._send_event_to_listener(create_event, self.images_index)

        create_event = self.image_events["image.member.create"]
        self._send_event_to_listener(create_event, self.images_index)
        result = self._verify_event_processing(create_event,
                                               owner=test_listener.OWNER1)

        # member.create event will have status of "pending" which should not
        # add the member to the image.members list
        self.assertEqual(0,
                         len(result['hits']['hits'][0]['_source']['members']))

    def test_image_member_update_event(self):
        """Send member.update notification event to listener"""

        create_event = self.image_events["image.create"]
        self._send_event_to_listener(create_event, self.images_index)

        update_event = self.image_events["image.member.update"]
        self._send_event_to_listener(update_event, self.images_index)
        result = self._verify_event_processing(update_event,
                                               owner=test_listener.OWNER1)

        # member.update event with the status of "accepted" which should
        # add the member to the image.members list
        self.assertEqual(1,
                         len(result['hits']['hits'][0]['_source']['members']))

    def test_image_member_delete_event(self):
        """Send member.delete notification event to listener"""

        create_event = self.image_events["image.create"]
        self._send_event_to_listener(create_event, self.images_index)

        update_event = self.image_events["image.member.update"]
        self._send_event_to_listener(update_event, self.images_index)
        result = self._verify_event_processing(update_event,
                                               owner=test_listener.OWNER1)
        self.assertEqual(1,
                         len(result['hits']['hits'][0]['_source']['members']))

        delete_event = self.image_events["image.member.delete"]
        self._send_event_to_listener(delete_event, self.images_index)
        result = self._verify_event_processing(delete_event,
                                               owner=test_listener.OWNER1)

        # member.delete event should remove the member
        self.assertEqual(0,
                         len(result['hits']['hits'][0]['_source']['members']))

    def test_md_namespace_create_event(self):
        """Send metadef_namespace.create notification event to listener"""
        create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(create_event, self.metadefs_index)
        result = self._verify_event_processing(create_event)
        verification_keys = ['namespace', 'display_name']
        self._verify_result(create_event, verification_keys, result)

    def test_md_namespace_update_event(self):
        """Send metadef_namespace.update notification event to listener"""

        create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(create_event, self.metadefs_index)

        update_event = self.metadef_events["metadef_namespace.update"]
        self._send_event_to_listener(update_event, self.metadefs_index)
        result = self._verify_event_processing(update_event)
        verification_keys = ['display_name', 'description', 'namespace',
                             'created_at', 'updated_at', 'owner', 'visibility',
                             'protected']
        self._verify_result(update_event, verification_keys, result)

    def test_md_namespace_update_conflict(self):
        """Send an outdated metadef_namespace.update notification event to
        listener, test if the document will be updated
        """
        create_event = self.metadef_events['metadef_namespace.create']
        self._send_event_to_listener(create_event, self.metadefs_index)
        update_event = self.metadef_events['metadef_namespace.update']
        self._send_event_to_listener(update_event, self.metadefs_index)

        outdate_update_event = copy.deepcopy(update_event)
        outdate_update_event['payload']['updated_at'] = '2015-09-01T03:51:54Z'
        self._send_event_to_listener(outdate_update_event, self.metadefs_index)
        result = self._verify_event_processing(outdate_update_event)

        # check if outdated update event failed
        verification_keys = ['updated_at']
        self._verify_result(update_event, verification_keys, result)

    def test_md_namespace_delete_event(self):
        """Send metadef_namespace.delete notification event to listener"""

        create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(create_event, self.metadefs_index)
        self._verify_event_processing(create_event)

        delete_event = self.metadef_events["metadef_namespace.delete"]
        self._send_event_to_listener(delete_event, self.metadefs_index)
        self._verify_event_processing(delete_event, 0)

    def test_md_object_create_event(self):
        """Send metadef_object.create notification event to listener"""
        ns_create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(ns_create_event, self.metadefs_index)

        obj_create_event = self.metadef_events["metadef_object.create"]
        self._send_event_to_listener(obj_create_event, self.metadefs_index)
        result = self._verify_event_processing(
            obj_create_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['objects']
        self.assertEqual(1, len(result))
        self.assertEqual(obj_create_event['payload']['name'],
                         result[0]['name'])

    def test_md_object_update_event(self):
        """Send metadef_object.update notification event to listener"""
        ns_create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(ns_create_event, self.metadefs_index)

        obj_create_event = self.metadef_events["metadef_object.create"]
        self._send_event_to_listener(obj_create_event, self.metadefs_index)

        obj_update_event = self.metadef_events["metadef_object.update"]
        self._send_event_to_listener(obj_update_event, self.metadefs_index)
        result = self._verify_event_processing(
            obj_update_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['objects'][0]
        self.assertEqual(2, len(result['properties']))

    def test_md_object_delete_event(self):
        """Send metadef_object.delete notification event to listener"""
        ns_create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(ns_create_event, self.metadefs_index)

        obj_create_event = self.metadef_events["metadef_object.create"]
        self._send_event_to_listener(obj_create_event, self.metadefs_index)
        result = self._verify_event_processing(
            obj_create_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['objects']
        self.assertEqual(1, len(result))

        obj_delete_event = self.metadef_events["metadef_object.delete"]
        self._send_event_to_listener(obj_delete_event, self.metadefs_index)
        result = self._verify_event_processing(
            obj_delete_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['objects']
        self.assertEqual(0, len(result))

    def test_md_property_create_event(self):
        """Send metadef_property.create notification event to listener"""
        ns_create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(ns_create_event, self.metadefs_index)

        prop_create_event = self.metadef_events["metadef_property.create"]
        self._send_event_to_listener(prop_create_event, self.metadefs_index)
        result = self._verify_event_processing(
            prop_create_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['properties']
        self.assertEqual(1, len(result))
        self.assertEqual(prop_create_event['payload']['type'],
                         result[0]['type'])

    def test_md_property_update_event(self):
        """Send metadef_property.update notification event to listener"""
        ns_create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(ns_create_event, self.metadefs_index)

        prop_update_event = self.metadef_events["metadef_property.update"]
        self._send_event_to_listener(prop_update_event, self.metadefs_index)
        result = self._verify_event_processing(
            prop_update_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['properties']
        self.assertEqual(1, len(result))
        self.assertEqual(prop_update_event['payload']['maximum'],
                         result[0]['maximum'])

    def test_md_property_delete_event(self):
        """Send metadef_object.delete notification event to listener"""
        ns_create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(ns_create_event, self.metadefs_index)

        prop_create_event = self.metadef_events["metadef_property.create"]
        self._send_event_to_listener(prop_create_event, self.metadefs_index)
        result = self._verify_event_processing(
            prop_create_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['properties']
        self.assertEqual(1, len(result))

        prop_delete_event = self.metadef_events["metadef_property.delete"]
        self._send_event_to_listener(prop_delete_event, self.metadefs_index)
        result = self._verify_event_processing(
            prop_delete_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['properties']
        self.assertEqual(0, len(result))

    def test_md_resource_type_create_event(self):
        """Send metadef_property.create notification event to listener"""
        ns_create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(ns_create_event, self.metadefs_index)

        res_type_create_event = \
            self.metadef_events["metadef_resource_type.create"]
        self._send_event_to_listener(res_type_create_event,
                                     self.metadefs_index)
        result = self._verify_event_processing(
            res_type_create_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['resource_types']
        self.assertEqual(1, len(result))
        self.assertEqual(res_type_create_event['payload']['name'],
                         result[0]['name'])

    def test_md_resource_type_delete_event(self):
        """Send metadef_property.create notification event to listener"""
        ns_create_event = self.metadef_events["metadef_namespace.create"]
        self._send_event_to_listener(ns_create_event, self.metadefs_index)

        res_type_create_event = \
            self.metadef_events["metadef_resource_type.create"]
        self._send_event_to_listener(res_type_create_event,
                                     self.metadefs_index)
        result = self._verify_event_processing(
            res_type_create_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['resource_types']
        self.assertEqual(1, len(result))

        rs_type_delete_event = \
            self.metadef_events["metadef_resource_type.delete"]
        self._send_event_to_listener(rs_type_delete_event, self.metadefs_index)
        result = self._verify_event_processing(
            rs_type_delete_event,
            owner=ns_create_event['payload']['owner'])
        result = result['hits']['hits'][0]['_source']['resource_types']
        self.assertEqual(0, len(result))


class TestGlanceLoad(functional.FunctionalTest):

    def setUp(self):
        super(TestGlanceLoad, self).setUp()

        openstack_client_mod = "searchlight.elasticsearch.plugins." \
                               "openstack_clients.get_glanceclient"
        osclient_patcher = mock.patch(
            openstack_client_mod, mock_glance_pyclient.get_fake_glance_client
        )
        osclient_patcher.start()
        self.addCleanup(osclient_patcher.stop)

        self.images_count, self.images_owner = \
            self._get_glance_image_owner_and_count()
        self.metadefs_count, self.metadefs_owner = \
            self._get_glance_metadefs_owner_and_count()
        self.all_doc_count = self.images_count + self.metadefs_count

        self.images_plugin = self.initialized_plugins['OS::Glance::Image']
        self.metadefs_plugin = self.initialized_plugins['OS::Glance::Metadef']

    def _get_glance_image_owner_and_count(self):
        with open(generate_load_data.IMAGES_FILE, "r+b") as file:
            images_data = jsonutils.load(file)
        if len(images_data) > 0:
            return len(images_data), images_data[0]['owner']

    def _get_glance_metadefs_owner_and_count(self):
        with open(generate_load_data.METADEFS_FILE, "r+b") as file:
            metadefs_data = jsonutils.load(file)
        if len(metadefs_data) > 0:
            return len(metadefs_data), metadefs_data[0]['owner']

    def test_searchlight_glance_images_data(self):
        """Test that all the indexed images data is served from api server"""

        self.images_plugin.index_initial_data()
        self._flush_elasticsearch(self.images_plugin.alias_name_search)
        glance_images_query = test_api.MATCH_ALL.copy()
        glance_images_query['index'] = self.images_plugin.alias_name_search
        glance_images_query['type'] = self.images_plugin.get_document_type()
        response, json_content = self._search_request(
            glance_images_query,
            self.images_owner)
        self.assertEqual(self.images_count,
                         json_content['hits']['total'])

    def test_searchlight_glance_metadefs_data(self):
        """Test that all the indexed metadefs data is served from api server"""
        self.metadefs_plugin.index_initial_data()
        self._flush_elasticsearch(self.metadefs_plugin.alias_name_search)
        metadefs_query = test_api.MATCH_ALL.copy()
        metadefs_query['index'] = self.metadefs_plugin.alias_name_search
        metadefs_query['type'] = self.metadefs_plugin.get_document_type()
        metadefs_query['sort'] = {'namespace': {'order': 'asc'}}
        response, json_content = self._search_request(metadefs_query,
                                                      self.metadefs_owner)
        self.assertEqual(self.metadefs_count,
                         json_content['hits']['total'])

    def test_es_all_data(self):
        """Test that all the data is indexed in elasticsearch server"""

        for plugin in self.images_plugin, self.metadefs_plugin:
            plugin.index_initial_data()
        self._flush_elasticsearch()
        # Test the raw elasticsearch response
        elasticsearch_docs = self._get_all_elasticsearch_docs()
        self.assertEqual(self.all_doc_count,
                         elasticsearch_docs['hits']['total'])



# Copyright (c) 2016 Hewlett-Packard Enterprise Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import mock

from searchlight.elasticsearch.plugins.cinder import serialize_cinder_snapshot
from searchlight.elasticsearch.plugins.cinder \
    import snapshots as snapshots_plugin
import searchlight.tests.unit.utils as unit_test_utils
import searchlight.tests.utils as test_utils


USER1 = u'1111bbbbccccc'
ID1 = u'1111-2222-3333'
VOLID1 = u'aaaa-cccc-bbbb'
PROJECT1 = u'aaabbbccc'


_now = datetime.datetime.utcnow()
_five_minutes_ago = _now - datetime.timedelta(minutes=5)
created_now = _five_minutes_ago.strftime(u'%Y-%m-%dT%H:%M:%S.%s')
updated_now = _now.strftime(u'%Y-%m-%dT%H:%M:%S.%s')


def _snapshot_fixture(snapshot_id, volume_id, project_id, **kwargs):
    fixture = {
        '_info': {
            u'created_at': created_now,
            u'description': None,
            # blah blah, don't want any of this
            u'volume_id': u'faaad6fe-9351-4313-bce9-881b476d5751'},
        '_loaded': True,
        'created_at': created_now,
        'description': None,
        'id': snapshot_id,
        'manager': "we don't want this",
        'metadata': {},
        'name': None,
        'os-extended-snapshot-attributes:progress': u'100%',
        'os-extended-snapshot-attributes:project_id': project_id,
        'size': 1,
        'status': u'available',
        'updated_at': updated_now,
        'volume_id': volume_id}

    return test_utils.DictObj(**fixture)


class TestSnapshotPlugin(test_utils.BaseTestCase):
    def setUp(self):
        super(TestSnapshotPlugin, self).setUp()
        self.plugin = snapshots_plugin.SnapshotIndex()
        self.snapshot1 = _snapshot_fixture(ID1, VOLID1, PROJECT1, name="snap1")

    def test_document_type(self):
        self.assertEqual('OS::Cinder::Snapshot',
                         self.plugin.get_document_type())

    def test_parent_type(self):
        self.assertEqual('OS::Cinder::Volume',
                         self.plugin.parent_plugin_type())

    def test_rbac_filters(self):
        fake_request = unit_test_utils.get_fake_request(
            USER1, PROJECT1, '/v1/search/facets', is_admin=True
        )
        rbac_query_fragment = self.plugin._get_rbac_field_filters(
            fake_request.context)

        self.assertEqual([{"term": {"project_id": PROJECT1}}],
                         rbac_query_fragment)

    def test_serialize(self):
        with mock.patch('cinderclient.v2.volume_snapshots.SnapshotManager.get',
                        return_value=self.snapshot1):
            serialized = serialize_cinder_snapshot(ID1)
        self.assertNotIn('_info', serialized)
        self.assertNotIn('_loaded', serialized)
        self.assertNotIn('manager', serialized)

        self.assertEqual(serialized['tenant_id'], PROJECT1)
        self.assertEqual(serialized['project_id'], PROJECT1)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from searchlight.elasticsearch.plugins import base
from searchlight.elasticsearch.plugins.ironic import obj_payload
from searchlight.elasticsearch.plugins.ironic import resources
from searchlight.elasticsearch.plugins.ironic import serialize_resource
from searchlight.elasticsearch.plugins.ironic import versioned_payload
from searchlight import pipeline

LOG = logging.getLogger(__name__)


class NodeHandler(base.NotificationBase):

    def __init__(self, *args, **kwargs):
        self.port_helper = kwargs.pop('port_helper')
        super(NodeHandler, self).__init__(*args, **kwargs)

    @classmethod
    def _get_notification_exchanges(cls):
        return ['ironic']

    def get_event_handlers(self):
        return {
            'baremetal.node.power_set.end': self.node_update,
            'baremetal.node.power_set.error': self.node_update,
            'baremetal.node.power_state_corrected.success': self.node_update,

            'baremetal.node.provision_set.start': self.node_update,
            'baremetal.node.provision_set.end': self.node_update,
            'baremetal.node.provision_set.error': self.node_update,
            'baremetal.node.provision_set.success': self.node_update,

            'baremetal.node.create.end': self.node_create_update,
            'baremetal.node.update.end': self.node_create_update,
            'baremetal.node.delete.end': self.node_delete,

            'baremetal.node.maintenance_set.end': self.node_update,

            'baremetal.node.console_set.end': self.node_update,
            'baremetal.node.console_set.error': self.node_update,
            'baremetal.node.console_restore.error': self.node_update
        }

    def get_log_fields(self, event_type, payload):
        return (
            ('version', payload.get('ironic_object.version')),
            ('id', obj_payload(payload)['uuid'])
        )

    @versioned_payload
    def node_create_update(self, event_type, payload, timestamp):
        LOG.debug("Updating node information for %s", payload['uuid'])
        node = serialize_resource(payload, resources.NODE_FIELDS)
        version = self.get_version(node, timestamp)
        self.index_helper.save_document(node, version=version)
        return pipeline.IndexItem(self.index_helper.plugin, event_type,
                                  payload, node)

    @versioned_payload
    def node_update(self, event_type, payload, timestamp):
        node_id = payload['uuid']
        LOG.debug("Updating node information for %s", node_id)
        node = serialize_resource(payload, resources.NODE_FIELDS)
        self.index_helper.update_document(node,
                                          node_id,
                                          update_as_script=False)
        return pipeline.IndexItem(self.index_helper.plugin, event_type,
                                  payload, node)

    @versioned_payload
    def node_delete(self, event_type, payload, timestamp):
        node_id = payload['uuid']
        LOG.debug("Deleting node %s", node_id)

        try:
            ports = self.port_helper.delete_documents_with_parent(node_id)
            deleted = [pipeline.DeleteItem(self.port_helper.plugin,
                                           event_type, payload,
                                           port['_id']) for port in ports]
            self.index_helper.delete_document({'_id': node_id})
            deleted.append(pipeline.DeleteItem(self.index_helper.plugin,
                                               event_type, payload, node_id))
            return deleted
        except Exception as exc:
            LOG.error(
                'Error deleting node %(node_id)s '
                'from index. Error: %(exc)s' %
                {'node_id': node_id, 'exc': exc})


class PortHandler(base.NotificationBase):
    @classmethod
    def _get_notification_exchanges(cls):
        return ['ironic']

    def get_event_handlers(self):
        return {
            'baremetal.port.create.end': self.port_create_update,
            'baremetal.port.update.end': self.port_create_update,
            'baremetal.port.delete.end': self.port_delete,
        }

    def get_log_fields(self, event_type, payload):
        return (
            ('version', payload.get('ironic_object.version')),
            ('id', obj_payload(payload)['uuid'])
        )

    @versioned_payload
    def port_create_update(self, event_type, payload, timestamp):
        LOG.debug("Updating port information for %s", payload['uuid'])
        port = serialize_resource(payload, resources.PORT_FIELDS)
        version = self.get_version(port, timestamp)
        self.index_helper.save_document(port, version=version)
        return pipeline.IndexItem(self.index_helper.plugin, event_type,
                                  payload, port)

    @versioned_payload
    def port_delete(self, event_type, payload, timestamp):
        port_id = payload['uuid']
        LOG.debug("Deleting port %s", port_id)
        try:
            self.index_helper.delete_document_unknown_parent(port_id)
            return pipeline.DeleteItem(self.index_helper.plugin, event_type,
                                       payload, port_id)
        except Exception as exc:
            LOG.error(
                'Error deleting port %(port_id)s '
                'from index. Error: %(exc)s' %
                {'port_id': port_id, 'exc': exc})


class ChassisHandler(base.NotificationBase):
    @classmethod
    def _get_notification_exchanges(cls):
        return ['ironic']

    def get_event_handlers(self):
        return {
            'baremetal.chassis.create.end': self.chassis_create_update,
            'baremetal.chassis.update.end': self.chassis_create_update,
            'baremetal.chassis.delete.end': self.chassis_delete,
        }

    def get_log_fields(self, event_type, payload):
        return (
            ('version', payload.get('ironic_object.version')),
            ('id', obj_payload(payload)['uuid'])
        )

    @versioned_payload
    def chassis_create_update(self, event_type, payload, timestamp):
        LOG.debug("Updating chassis information for %s", payload['uuid'])
        chassis = serialize_resource(payload, resources.CHASSIS_FIELDS)
        version = self.get_version(chassis, timestamp)
        self.index_helper.save_document(chassis, version=version)
        return pipeline.IndexItem(self.index_helper.plugin, event_type,
                                  payload, chassis)

    @versioned_payload
    def chassis_delete(self, event_type, payload, timestamp):
        chassis_id = payload['uuid']
        LOG.debug("Deleting chassis %s", chassis_id)
        try:
            self.index_helper.delete_document({'_id': chassis_id})
            return pipeline.DeleteItem(self.index_helper.plugin, event_type,
                                       payload, chassis_id)
        except Exception as exc:
            LOG.error(
                'Error deleting chassis %(chassis_id)s '
                'from index. Error: %(exc)s' %
                {'chassis_id': chassis_id, 'exc': exc})



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from oslo_serialization import jsonutils

from glanceclient.v2 import client as glance
from keystoneauth1 import session
from keystoneclient.auth.identity import v3
import novaclient.client

IMAGES_FILE = "searchlight/tests/functional/data/load/images.json"
METADEFS_FILE = "searchlight/tests/functional/data/load/metadefs.json"
IMAGE_MEMBERS_FILE = \
    "searchlight/tests/functional/data/load/image_members.json"
SERVERS_FILE = "searchlight/tests/functional/data/load/servers.json"
FLAVORS_FILE = "searchlight/tests/functional/data/load/flavors.json"
SERVER_GROUP_FILE = \
    "searchlight/tests/functional/data/load/server_groups.json"

_session = None


def _get_flavor_tenant(flavor):
    if flavor.is_public:
        return ""
    n_client = get_novaclient()
    flavor_access = n_client.flavor_access.list(flavor=flavor)[0]
    tenant_id = flavor_access.tenant_id
    return tenant_id


def _get_session():

    global _session
    if not _session:
        auth = v3.Password(auth_url=os.environ.get('OS_AUTH_URL'),
                           username=os.environ.get('OS_USERNAME'),
                           password=os.environ.get('OS_PASSWORD'),
                           tenant_name=os.environ.get('OS_TENANT_NAME'))
        ks_session = session.Session(auth=auth,
                                     verify=os.environ.get('OS_CACERT'))
    return ks_session


def get_glanceclient():
    session = _get_session()

    return glance.Client(
        session=session,
        region_name=os.environ.get('OS_REGION_NAME')
    )


def get_novaclient():
    session = _get_session()

    return novaclient.client.Client(
        version=2,
        session=session,
        region_name=os.environ.get('OS_REGION_NAME')
    )


def get_glance_images_and_members_with_pyclient():

    glance_client = get_glanceclient()
    images = glance_client.images.list()
    images_json = jsonutils.dumps(list(images), indent=4)
    with open(IMAGES_FILE, "w") as f:
        f.write(images_json)

    image_members_dict = dict()
    images = glance_client.images.list()
    for image in images:
        if image['visibility'] != 'public':
            image_members = glance_client.image_members.list(image['id'])
            image_members_list = []
            if image_members:
                image_members_list = list(image_members)
            if len(image_members_list) > 0:
                image_members_dict[image['id']] = image_members_list
    image_members_json = jsonutils.dumps(image_members_dict, indent=4)
    with open(IMAGE_MEMBERS_FILE, "w") as f:
        f.write(image_members_json)


def get_glance_metadefs_with_pyclient():

    glance_client = get_glanceclient()
    namespace_list = []
    metadefs_namespace_list = list(glance_client.metadefs_namespace.list())

    for namespace in metadefs_namespace_list:
        _namespace = glance_client.metadefs_namespace.get(
            namespace['namespace'])
        namespace_list.append(_namespace)

    metadef_namespace_json = jsonutils.dumps(namespace_list, indent=4)

    with open(METADEFS_FILE, "w") as f:
        f.write(metadef_namespace_json)


def get_nova_servers_with_pyclient():

    nova_client = get_novaclient()
    servers = nova_client.servers.list()
    servers_list = []
    for each in servers:
        servers_list.append(each.to_dict())
    servers_json = jsonutils.dumps(list(servers_list), indent=4)
    with open(SERVERS_FILE, "w") as f:
        f.write(servers_json)


def get_nova_flavors_with_pyclient():

    nova_client = get_novaclient()
    flavor = nova_client.flavors.list()[0]
    flavor_dict = flavor.to_dict()
    flavor_dict.pop("links")
    flavor_dict.update({"tenant_id": _get_flavor_tenant(flavor)})
    flavor_dict.update({"extra_spec": flavor.get_keys()})
    flavors_json = jsonutils.dumps([flavor_dict], indent=4)
    with open(FLAVORS_FILE, "w") as f:
        f.write(flavors_json)


def get_nova_server_groups_with_pyclient():

    nova_client = get_novaclient()
    server_groups = nova_client.server_groups.list(all_projects=True)
    server_groups_list = []
    for each in server_groups:
        server_groups_list.append(each.to_dict())
        server_groups_json = jsonutils.dumps(list(server_groups_list),
                                             indent=4)
    with open(SERVERS_FILE, "w") as f:
        f.write(server_groups_json)


def generate():
    get_glance_images_and_members_with_pyclient()
    get_glance_metadefs_with_pyclient()
    get_nova_servers_with_pyclient()
    get_nova_flavors_with_pyclient()
    get_nova_server_groups_with_pyclient()

if __name__ == "__main__":
    generate()
